#define M_PI        3.14159265358979323846

void ReceivePacket(struct demod_state *s, unsigned char *bp, unsigned int len);
void hdlc_init(struct demod_state *s);
void hdlc_rxbit(struct demod_state *s, int bit);
void pocsag_init(struct demod_state *s);
void pocsag_rxbit(struct demod_state *s, int bit);
void fsk96_demod(struct demod_state *s, float *buffer, int length);
void afsk24_demod(struct demod_state *s, float *buffer, int length);
void afsk12_demod(struct demod_state *s, float *buffer, int length);
void hapn48_demod(struct demod_state *s, float *buffer, int length);
void afsk24_2_demod(struct demod_state *s, float *buffer, int length);
void poc5_demod(struct demod_state *s, float *buffer, int length);
void poc24_demod(struct demod_state *s, float *buffer, int length);
void poc12_demod(struct demod_state *s, float *buffer, int length);
void fsk96_init(struct demod_state *s);
void afsk12_init(struct demod_state *s);
void afsk24_init(struct demod_state *s);
void afsk24_2_init(struct demod_state *s);
void hapn48_init(struct demod_state *s);
void poc5_init(struct demod_state *s);
void poc24_init(struct demod_state *s);
void poc12_init(struct demod_state *s);

float fsqr(float f);
float mac(const float *a, const float *b, unsigned int size);

struct demod_state {
	const struct demod_param *dem_par;
	union {
		struct l2_state_hdlc {
			unsigned char rxbuf[512];
			unsigned char *rxptr;
			unsigned int rxstate;
			unsigned int rxbitstream;
			unsigned int rxbitbuf;
		} hdlc;

		struct l2_state_pocsag {
			unsigned long rx_data;
			struct l2_pocsag_rx {
				unsigned char rx_sync;
				unsigned char rx_word;
				unsigned char rx_bit;
				char func;
				unsigned long adr;
				unsigned char buffer[128];
				unsigned int numnibbles;
			} rx[2];
		} pocsag;
	} l2;
	union {
		struct l1_state_poc5 {
			unsigned int dcd_shreg;
			unsigned int sphase;
			unsigned int subsamp;
		} poc5;

		struct l1_state_poc12 {
			unsigned int dcd_shreg;
			unsigned int sphase;
			unsigned int subsamp;
		} poc12;

		struct l1_state_poc24 {
			unsigned int dcd_shreg;
			unsigned int sphase;
		} poc24;

		struct l1_state_afsk12 {
			unsigned int dcd_shreg;
			unsigned int sphase;
			unsigned int lasts;
			unsigned int subsamp;
		} afsk12;

		struct l1_state_afsk24 {
			unsigned int dcd_shreg;
			unsigned int sphase;
			unsigned int lasts;
		} afsk24;

		struct l1_state_hapn48 {
			unsigned int shreg;
			unsigned int sphase;
			float lvllo, lvlhi;
		} hapn48;

		struct l1_state_fsk96 {
			unsigned int dcd_shreg;
			unsigned int sphase;
			unsigned int descram;
		} fsk96;

		struct l1_state_dtmf {
			unsigned int ph[8];
			float energy[4];
			float tenergy[4][16];
			int blkcount;
			int lastch;
		} dtmf;

		struct l1_state_zvei {
			unsigned int ph[16];
			float energy[4];
			float tenergy[4][32];
			int blkcount;
			int lastch;
		} zvei;

		struct l1_state_scope {
			int datalen;
			int dispnum;
			float data[512];
		} scope;
	} l1;
};

struct demod_param {
	const char *name;
	unsigned int samplerate;
	unsigned int overlap;
	void (*init)(struct demod_state *s);
	void (*demod)(struct demod_state *s, float *buffer, int length);
};

