/*
 *
 *      setup.c v2.01b By Mikko Antero Gustafsson
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <io.h>
#include <direct.h>
#include <malloc.h>
#include <memory.h>
#include <shellapi.h>
#include <shlobj.h>
#include <wingdi.h>
#include <cpl.h>
#include <ole2.h>
#include <objbase.h>
#include <tlhelp32.h>
#include "setup.h"

const char *szAppName = "WinAFSK Setup";
const char *szApplicationClassName = "WinAFSK";

#define FILECOUNT 27
char *FileList[FILECOUNT] =
{
  "WinAFSK.exe", 
  "WinAFSK.chm", 
  "license.txt",
  "src\\winafdmd.c",
  "src\\winafsk.c",  
  "src\\winafsk.h",  
  "src\\winafsk.rc",
  "src\\winafsk.ico",
  "src\\winafsml.ico",
  "src\\buttons.bmp",
  "src\\makefile",
  "src\\setup\\setup.c",
  "src\\setup\\makefile",
  "src\\setup\\resource.h",
  "src\\setup\\setup.rc",
  "src\\setup\\setup.bmp",
  "src\\setup\\setup.h",
  "src\\setup\\logo.bmp",
  "src\\setup\\setup.ico",
  "src\\demod\\afsk12.c",
  "src\\demod\\afsk24.c",
  "src\\demod\\afsk24_2.c",
  "src\\demod\\hapn48.c",
  "src\\demod\\fsk96.c",
  "src\\demod\\hdlc.c",
  "src\\demod\\demod.h",
  "src\\demod\\filter.c"
};

#define MAX_BUFFER  1024
#define StartWait() hcurSave = SetCursor(LoadCursor(NULL,IDC_WAIT))
#define EndWait()   SetCursor(hcurSave)

//
//  A macro used for hiding and displaying the controls on the dialog window
//
#define Display(a,b,c)                           \
                                                 \
                {                                \
                HWND hWndTmp = GetDlgItem(a, b); \
                ShowWindow(hWndTmp,c);           \
                }
//
// Macros for determining operating system version
//
#if defined (WIN32)
   #define IS_WIN32 TRUE
#else
   #define IS_WIN32 FALSE
#endif
#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

//
//  used the dlgNumber variable to track the current dialog activity 
//
#define dlgNONE       -1
#define dlgSTART      0
#define dlgINSTALL    1
#define dlgPROGRESS   2
#define dlgCOMPLETE   3
#define dlgUNINSTALL  4

typedef struct _MYDATA
{
  char szTest1[80];       // a test buffer containing the file selected
	char szTest2[80];       // a test buffer containing the file path
} MYDATA, FAR * LPMYDATA;

char *WinAFSKdir = "\\WinAFSK";

//
//  Used for setting focus to the program, if the user tries to open
//  more than one instance. It returns the hWnd of the dialog box.
//  We need this because if the background window is invisible
//  we need to SetForegroundWindow(hDlg)
//
#define WM_RELOADING (WM_USER+1)

//
//  Used to hide the background window
//
#define HIDE_MAIN_WINDOW 1

char        bSystemCharset;
char        szTitle[100];        
char        szBuffer[MAX_BUFFER];
char        szTmpDir[MAX_BUFFER];
char        szWinDir[MAX_BUFFER];
char        szHomeDir[MAX_BUFFER];
char        szInstallDir[MAX_BUFFER];
char        szStartMenuDir[MAX_BUFFER];
char        szDesktopDir[MAX_BUFFER];
char       *iniDirSection = "Directories";
char       *iniDirKey = "HomeDirectory";
char       *pszInstallCancelMsg = "The Installation is not complete. If you terminate the\ninstallation now, the program will not be installed.\n\nDo you wish to terminate the installation?";
char       *pszUninstallCancelMsg = "Do you want to cancel the uninstallation";
char       *pszExeIsRunningMsg = "The WinAFSK program is currently running. You must first close it to perform the %sinstallation.";
char       *pszAlreadyInstalledMessage = "A previous installation of WinAFSK exists in that directory. The database will be removed.\r\nDo you wish to proceed with the installation?";
char        dlgInstallDir[1048];
int         dlgNumber=dlgSTART;
int         iBytesPerStep = 0;
int         iBytesCopied = 0;
int         iFileCount = 0;
int         msgDialog;
int         DiskSpaceRequired[2];  // 0 is the Windows dir, 1 is the install dir
BOOL        bCopyCancelFlag = FALSE;
BOOL        bUnInstallFlag = FALSE;
HWND        hCheck1, hCheck2;
HWND        mainhWnd = NULL;
HWND        hDialog = NULL;
HWND        hWndText;
HWND        hWndProgressBar;
HFONT       hFont = NULL;
WNDPROC     originalTextProc;
HINSTANCE   hInst;
HICON       hIcon;

void        QuitInstall(void);
void        ReadRegistry(void);
void        DeleteRegistry(void);
BOOL        RegisterProgram(char *szProgDir);
LRESULT     VerifiedCopyFile(HWND hWnd, char *source, char *destination, BOOL recurseflag);
LRESULT     VerifiedDeleteFile(HWND hWnd, char *destination);
LRESULT     CreateDir(char *dirName);
LRESULT     FileCopy(HWND hDialog, char *szInputFile, char *szOutputFile);
BOOL        CenterWindow (HWND hwndChild, HWND hwndParent, int addx, int addy);
void        EnsurePathName(char *PathAndFile);
void        GetSystemDirectories(char *ProgramFilesDir, char *StartMenuDir);
HRESULT     CreateLink(LPCSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc);
int         CheckFileSpace(char *target);
void        RemoveDoubleBackslash(char *buf);
void        ConvertSlashes(char *buf);
BOOL        CheckForInstallFiles(void);

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT APIENTRY TextProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK DialogCB(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK BrowseCB(HWND, UINT, WPARAM, LPARAM);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
  HWND hWnd;
  MSG msg;
  WNDCLASSEX WndClassEx;
  char szSource[MAX_BUFFER], szDest[MAX_BUFFER];
  BOOL bRet;
  STARTUPINFO sui;
  PROCESS_INFORMATION pi;

  if(!IS_NT)
  {
    MessageBox(GetFocus(), "WinAFSK cannot be installed on Windows 95, 98 or ME.", szAppName, MB_OK);
    exit(1);
  }

  if(hWnd = FindWindow(szAppName, NULL))
  {
    hWnd = (HWND)SendMessage(hWnd, WM_RELOADING, 0, 0);

    if (IsIconic(hWnd)) 
        ShowWindow(hWnd, SW_RESTORE);
    SetForegroundWindow(hWnd);

    return TRUE;
  }

  hInst = hInstance;

  //
  //  Set up an icon
  //
  hIcon = LoadIcon(hInstance, IDI_APPLICATION);

  //
  //  Initialize global strings
  //
  LoadString(hInstance, IDS_APP_TITLE, szTitle, 100);
  GetWindowsDirectory(szWinDir, MAX_BUFFER-2);
  GetSystemDirectories(szInstallDir, szStartMenuDir);
  ReadRegistry();

  GetModuleFileName(NULL, szSource, MAX_BUFFER);
  strcpy(szDest, szHomeDir);
  strcat(szDest, "\\UNINSTALL.EXE");

  if(stricmp(szDest, szSource)==0 || stricmp(lpCmdLine,"UNINSTALL")==0)
    bUnInstallFlag = TRUE;

  //
  //  The program makes a copy of itself, in the TEMP directory, and executes
  //  it. This way it is able to delete the whole install directory, including
  //  itself.
  //
  strcpy(szDest, szTmpDir);
  strcat(szDest, "P12SETUP.EXE");
  GetModuleFileName(NULL, szSource, MAX_BUFFER);

  if(bUnInstallFlag && stricmp(szDest, szSource)!=0)
  {
    CopyFile(szSource, szDest, FALSE);

    sui.cb = sizeof(STARTUPINFO);
    GetStartupInfo(&sui);
    sui.dwFlags = STARTF_USESHOWWINDOW;
    sui.wShowWindow = SW_SHOW;

    strcpy(szSource, szDest);
    strcat(szSource, " UNINSTALL");
    bRet = CreateProcess (szDest, szSource, NULL, NULL, FALSE,
                          0, NULL, szTmpDir, &sui, &pi);

    //
    //  For some reason ShellExecute() would cause the first EXE to remain
    //  open, and impossible to delete
    //
    //  ShellExecute(NULL,"open",szDest,"UNINSTALL",szTmpDir,SW_SHOWNORMAL);
    //

    PostQuitMessage(0);
    return TRUE;
  }

  //
  //  This fixes the the notorious focus issues associated with the
  //  UnInstall dialog in Windows. It is necessary, in order to fix it.
  //
  Sleep(500);

  if(bUnInstallFlag)
    SetCurrentDirectory(szHomeDir);

  //
  //  Operation based on a main background window
  //
  WndClassEx.cbSize          = sizeof(WNDCLASSEX);
  WndClassEx.style           = CS_HREDRAW | CS_VREDRAW;
  WndClassEx.lpfnWndProc     = WndProc;
  WndClassEx.cbClsExtra      = 0;
  WndClassEx.cbWndExtra      = 0;
  WndClassEx.hInstance       = hInstance;
  WndClassEx.hIcon           = hIcon;
  WndClassEx.hCursor         = LoadCursor(hInstance, IDC_ARROW);
  WndClassEx.hbrBackground   = (HBRUSH)(COLOR_3DSHADOW+1);
  WndClassEx.lpszClassName   = szAppName;
  WndClassEx.hIconSm         = hIcon;

  RegisterClassEx(&WndClassEx);

  //
  // Create a main window, to cover the whole screen
  //
  mainhWnd = CreateWindow(szAppName, szTitle, WS_OVERLAPPEDWINDOW,// | WS_MINIMIZE,
                      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
                      NULL, NULL, hInstance, NULL);
  if (!mainhWnd)
    return (FALSE);

#ifndef HIDE_MAIN_WINDOW
  if(bUnInstallFlag)
    ShowWindow(mainhWnd, SW_HIDE);
  else
    ShowWindow(mainhWnd, nCmdShow);
  cx = GetSystemMetrics(SM_CXSCREEN);
  cy = GetSystemMetrics(SM_CYSCREEN);
  SetWindowPos(mainhWnd, HWND_NOTOPMOST, 0, 0, cx, cy - 24, 0);
  UpdateWindow(mainhWnd);
#else
  ShowWindow(mainhWnd, SW_MINIMIZE);
#endif

  //
  //  This triggers the the first dialog
  //
  SendMessage(mainhWnd, WM_COMMAND, 0L, 0L);

  while(GetMessage(&msg,NULL,0,0))
  {
    if(!IsDialogMessage(mainhWnd,&msg))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }         
  }

  return (msg.wParam);

}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  MESSAGES:
//
//  WM_COMMAND - process the application menu
//  WM_PAINT - Paint the main window
//  WM_DESTROY - post a quit message and return
//  WM_DISPLAYCHANGE - message sent to Plug & Play systems when the display changes
//  WM_RBUTTONDOWN - Right mouse click -- put up context menu here if appropriate
//  WM_NCRBUTTONUP - User has clicked the right button on the application's system menu
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  POINT pnt;

  switch (message)
  {
    case WM_COMMAND:
      // Perform application initialization:
      DialogBox(hInst, "DIALOG1", mainhWnd, (DLGPROC)DialogCB);
      break;

    case WM_NCRBUTTONUP: // RightClick on windows non-client area...
      if (IS_WIN95 && SendMessage(hWnd, WM_NCHITTEST, 0, lParam) == HTSYSMENU)
      {
        // The user has clicked the right button on the applications
        // 'System Menu'. Here is where you would alter the default
        // system menu to reflect your application. Notice how the
        // explorer deals with this. For this app, we aren't doing
        // anything
        return DefWindowProc(hWnd, message, wParam, lParam);
      }
      else
      {
        // Nothing we are interested in, allow default handling...
        return DefWindowProc(hWnd, message, wParam, lParam);
      }
      break;

    case WM_RBUTTONDOWN: // RightClick in windows client area...
      pnt.x = LOWORD(lParam);
      pnt.y = HIWORD(lParam);
      ClientToScreen(hWnd, (LPPOINT) &pnt);
      break;

    case WM_DISPLAYCHANGE: // Only comes through on plug'n'play systems
      {
        SIZE  szScreen;
        DWORD dwBitsPerPixel = (DWORD)wParam;
        szScreen.cx = LOWORD(lParam);
        szScreen.cy = HIWORD(lParam);
      }
      break;

    case WM_RELOADING:
      return (long)hDialog;

    default:
      return (DefWindowProc(hWnd, message, wParam, lParam));
  }
  return (0);
}

LRESULT CALLBACK DialogCB(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

  HWND hWndTmp;
  int lResult;
  int iTmp;
  int lRet;
  HANDLE hFile;
  HWND hwndPT;
  int processcomplete;
  char sourcefile[MAX_BUFFER];
  char destDirName[MAX_BUFFER];
  char ShortName[MAX_BUFFER];
  char destfile[MAX_BUFFER];
  
  switch (message)
  {
    case WM_TIMER:
      if(dlgNumber==dlgPROGRESS)
        SendMessage(hWndProgressBar, PBM_SETPOS, iBytesCopied/iBytesPerStep, 0);
      break;

    case WM_INITDIALOG:
      hDialog = hDlg;
      SetForegroundWindow(mainhWnd);
      SetForegroundWindow(hDlg);
      switch(dlgNumber)
      {
        case dlgSTART:
          Display(hDlg, IDC_CHECKBOX1, SW_HIDE);
          Display(hDlg, IDC_CHECKBOX2, SW_HIDE);
          Display(hDlg, IDC_PROGRESS1, SW_HIDE);

          if(bUnInstallFlag)
          {
            if(szHomeDir[0] == 0)
            {
              MessageBox(GetFocus(), "Cannot find WinAFSK's registration information", szAppName, MB_OK);
              QuitInstall();
              return FALSE;
            }
            strcpy(dlgInstallDir,szHomeDir);
            dlgNumber = dlgUNINSTALL;

            Display(hDlg, IDE_DIRECTORY, SW_HIDE);
            Display(hDlg, IDC_BROWSE, SW_HIDE);
            Display(hDlg, IDC_KKGROUPBOX, SW_HIDE);
            SetDlgItemText(hDlg, IDC_WELCOMEMSG, "Welcome to the WinAFSK uninstallation program. The WinAFSK program and the entire message database will now be removed from your computer.");
          }
          else
          {
            if(szHomeDir[0])
            {
              strcpy(szInstallDir, szHomeDir);
            }
            else
            {
              strcat(szInstallDir, WinAFSKdir);
              strcpy(szHomeDir, szInstallDir);
            }
            DiskSpaceRequired[0] = DiskSpaceRequired[1] = iTmp = 0;
          
            //
            //  determine the total diskspace required
            //
            for(iFileCount = 0; iFileCount < FILECOUNT; iFileCount++ )
            {
              hFile = CreateFile(FileList[iFileCount], GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
              if(hFile == NULL)
              {
                sprintf(szBuffer,"Cannot read the file - %s", FileList[iFileCount]);
                QuitInstall();
                return(FALSE);
              }
              DiskSpaceRequired[1] += GetFileSize(hFile, &iTmp);
              CloseHandle(hFile);
            }
            iBytesPerStep = DiskSpaceRequired[1] / 10000;
            bCopyCancelFlag = FALSE;
            strcpy(dlgInstallDir, szHomeDir);
            dlgNumber = dlgINSTALL;
            SetDlgItemText(hDlg, IDE_DIRECTORY, dlgInstallDir);
            hWndTmp = GetDlgItem(hDlg, IDC_KKGROUPBOX);
            ShowWindow(hWndTmp, SW_SHOWNORMAL);
          }
          break;

        case dlgPROGRESS:
          Display(hDlg, IDE_DIRECTORY, SW_HIDE);
          Display(hDlg, IDC_BROWSE, SW_HIDE);
          Display(hDlg, IDC_PROGRESS1, SW_SHOWNORMAL);
          Display(hDlg, IDC_KKGROUPBOX, SW_SHOWNORMAL);
          break;

        case dlgCOMPLETE:
          if(bUnInstallFlag)
          {
            Display(hDlg, IDC_KKGROUPBOX, SW_HIDE);
          }
          else
          {
            //
            //  get CheckBox hWnds, and set them to checked
            //
            hCheck1 = GetDlgItem(hDlg, IDC_CHECKBOX1);
            hCheck2 = GetDlgItem(hDlg, IDC_CHECKBOX2);
            ShowWindow(hCheck1, SW_SHOWNORMAL);
            ShowWindow(hCheck2, SW_SHOWNORMAL);
            lResult = SendMessage((HWND) hCheck1, (UINT) BM_SETCHECK, 1, 0);  
            lResult = SendMessage((HWND) hCheck2, (UINT) BM_SETCHECK, 1, 0);  
          }
          Display(hDlg,IDC_PROGRESS1,SW_HIDE);

          hWndTmp = GetDlgItem(hDlg, IDC_BUTTON2);
          SetWindowText(hWndTmp, "&Done");

          hWndTmp = GetDlgItem(hDlg, IDC_WELCOMEMSG);
          if(bUnInstallFlag)
            SetWindowText(hWndTmp, "The uninstallation of WinAFSK is complete.");
          else
            SetWindowText(hWndTmp, "The installation of WinAFSK is complete.");
          break;
      }
#ifdef HIDE_MAIN_WINDOW
      CenterWindow (hDlg, NULL, 0, 0);
#else
      CenterWindow (hDlg, mainhWnd, 0, 0);
#endif
      return (TRUE);

    case WM_COMMAND:
      switch(dlgNumber)
      {
        case dlgPROGRESS:
          if(MessageBox(mainhWnd, pszInstallCancelMsg, szAppName,
                       MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION) == 6)
          {
            msgDialog = IDC_BUTTON2;
            bCopyCancelFlag = TRUE;
            EndDialog(hDlg, TRUE);
          }
          return TRUE;

        case dlgCOMPLETE:
          if(LOWORD(wParam) == IDC_CHECKBOX1)
          {
            lResult = SendMessage((HWND) hCheck1,(UINT) BM_GETCHECK,0,0);  
            lResult = SendMessage((HWND) hCheck1,(UINT) BM_SETCHECK,!lResult,0);  
          }
          else
          {
            if(SendMessage((HWND) hCheck2,(UINT) BM_GETCHECK,0,0))
            {
              strcpy(sourcefile, szHomeDir);
              strcat(sourcefile, "\\WinAFSK.EXE");
              strcpy(destfile, szDesktopDir);
              strcat(destfile, "\\WinAFSK.LNK");
              CreateLink(sourcefile, destfile, "WinAFSK");
            }

            if(SendMessage((HWND) hCheck1,(UINT) BM_GETCHECK,0,0))
            {
              strcpy(szInstallDir, szHomeDir);
              strcat(szInstallDir, "\\WinAFSK.EXE");
              ShellExecute(NULL, "open",szInstallDir, "", szHomeDir, SW_SHOWNORMAL);
              Sleep(50);
              hWndTmp = FindWindow(szApplicationClassName, NULL);
              if(hWndTmp != NULL)
              {
                SetForegroundWindow(hWndTmp);
              }
            }
            QuitInstall();
            return TRUE;
          }
          break;
           
        case dlgINSTALL:
          switch(LOWORD(wParam))
          {
            case IDC_BROWSE:
              {
                LPITEMIDLIST hlp;
                BROWSEINFO bi;
                bi.hwndOwner      = hDlg;
                bi.pidlRoot       = NULL;
                bi.pszDisplayName = szBuffer;
                bi.lpszTitle      = "";
                bi.lpfn           = (BFFCALLBACK)BrowseCB;
                bi.lParam         = (LPARAM)&szHomeDir;
                bi.iImage         = 0;
                bi.ulFlags        = BIF_RETURNONLYFSDIRS;
                hlp = SHBrowseForFolder(&bi);
                if(hlp)
                {
                  strcpy(szHomeDir, dlgInstallDir);
                  SetDlgItemText(hDlg, IDE_DIRECTORY, dlgInstallDir);
                }
                else
                {
                  strcpy(dlgInstallDir, szHomeDir);
                }
              }
              break;

            case XCLOSEBUTTON:
            case IDC_BUTTON2:
              if(MessageBox(GetFocus(), pszInstallCancelMsg, szAppName,
                           MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION) == 6)
              {
                QuitInstall();
                return FALSE;
              }
              break;

            case IDC_BUTTON1:

              if(!CheckForInstallFiles())
              {
                PostQuitMessage(0);
                return FALSE;
              }

              GetDlgItemText(hDlg, IDE_DIRECTORY, dlgInstallDir, MAX_BUFFER-1);
              RemoveDoubleBackslash(dlgInstallDir);
              if((dlgInstallDir[1] != ':') && !(dlgInstallDir[0]=='\\' && dlgInstallDir[1]=='\\'))
              {
                MessageBox(GetFocus(), "You must specify a full path, including a storage device letter, or network share name.", szAppName, MB_OK);
                return (TRUE);
              }

              SetDlgItemText(hDlg, IDE_DIRECTORY, dlgInstallDir);
              if(!CheckFileSpace(dlgInstallDir))
                return (TRUE);

              if(GetFileAttributes(dlgInstallDir) == -1)
              {
                if(!CreateDir(dlgInstallDir))
                  return (TRUE);
              }
              else
              {
                wsprintf(szBuffer, "%s\\winafsk.DB", dlgInstallDir);
                if(GetFileAttributes(szBuffer) != -1)
                {
                  //
                  //  This SetFocus prevents any extra WM_COMMANDS to the
                  //  cancel button
                  //
                  SetFocus(GetDlgItem(hDlg, IDC_BUTTON1));
                  if(MessageBox(GetFocus(), pszAlreadyInstalledMessage, szAppName, 
                                MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION) != 6)
                    return (TRUE);
                }
              }

              hWndTmp = FindWindow(szApplicationClassName, NULL);
              if(hWndTmp != NULL)
              {
//                SendMessage(hWndTmp, WM_DESTROY, 0, 0);
                SendMessage(hWndTmp, WM_CLOSE, 0, 0);
                Sleep(500);

                hWndTmp = FindWindow(szApplicationClassName, NULL);
                if(hWndTmp != NULL)
                {
  
                  wsprintf(szBuffer, "%s\\WinAFSK.EXE", szHomeDir);
              
                  if(bUnInstallFlag)
                    wsprintf(szBuffer, pszExeIsRunningMsg, "un");
                  else
                    wsprintf(szBuffer, pszExeIsRunningMsg, "");
              
                  MessageBox(NULL, szBuffer, szAppName, MB_OK);
              
                  if (IsIconic(hWndTmp))
                    ShowWindow(hWndTmp, SW_RESTORE);
              
                  SetForegroundWindow(hWndTmp);
              
                  PostQuitMessage(0);
                  return FALSE;
                }
              }
          
              sprintf(szBuffer, "%s\\src", dlgInstallDir);
              if(!CreateDir(szBuffer))
                return (TRUE);

              sprintf(szBuffer, "%s\\src\\setup", dlgInstallDir);
              if(!CreateDir(szBuffer))
                return (TRUE);

              sprintf(szBuffer, "%s\\src\\demod", dlgInstallDir);
              if(!CreateDir(szBuffer))
                return (TRUE);

              //
              //  display the progress dialog
              //
              dlgNumber = dlgPROGRESS;
              SendMessage(hDlg, WM_INITDIALOG, 0, 0);
            
              strcpy(szHomeDir, dlgInstallDir);
              strcpy(destDirName, dlgInstallDir);
        
              //
              //  display the progress dialog, non-modally
              //
              hWndProgressBar = GetDlgItem(hDlg, IDC_PROGRESS1);
              hwndPT = GetDlgItem(hDlg, IDC_WELCOMEMSG);
        
              Display(hDlg,IDC_BUTTON1, SW_HIDE);

              hWndTmp = GetDlgItem(hDlg, IDC_KKGROUPBOX);
              SetWindowText(hWndTmp, "");

              SendMessage(hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0, 10000));
        
              ShowWindow(hDlg, SW_NORMAL);
              EnableWindow(mainhWnd, FALSE);
              SetTimer(hDlg, (int)NULL, 250, (int)NULL);

              //
              // install registration information
              //
              if(!RegisterProgram(dlgInstallDir))
              {
                MessageBox(mainhWnd, "The program registration process failed. If you are installing on Windows NT/XP/2000,\nlog in as Administrator, and then install this program.", szAppName, MB_OK);
                QuitInstall();
                return FALSE;
              }
        
              // copy the UINST.EXE file to the install directory
              sprintf(destfile, "%s\\UnInstall.exe", dlgInstallDir);
              if(!VerifiedCopyFile(hDlg, ".\\SETUP.EXE", destfile, FALSE))
              {
                if(bCopyCancelFlag)
                  return FALSE;
        
                QuitInstall();
                return TRUE;
              }
        
              hWndTmp = GetDlgItem(hDlg, IDC_BUTTON1);
              SetFocus(hWndTmp);
        
              processcomplete = 0;
              strcpy(destDirName, dlgInstallDir);
        
              for(iFileCount=0; iFileCount < FILECOUNT; iFileCount++)
              {
                strcpy(ShortName, FileList[iFileCount]);
                if(strlen(ShortName)>20)
                {
                  szBuffer[0] = szBuffer[1] = szBuffer[2] = '.';
                  szBuffer[3] = 0;
                  strcat(szBuffer, &ShortName[strlen(ShortName) - 23]);
                  strcpy(ShortName, szBuffer);
                }
                sprintf(szBuffer,"Installing file:   %s                ", ShortName);
                SetWindowText(hwndPT, szBuffer);
        
                sprintf(destfile, "%s\\%s", destDirName, FileList[iFileCount]);
                sprintf(sourcefile, ".\\%s", FileList[iFileCount]);
        
                if(!VerifiedCopyFile(hDlg, sourcefile, destfile, FALSE))
                {
                  if(bCopyCancelFlag)
                    return FALSE;
        
                  QuitInstall();
                  return TRUE;
                }
                
                processcomplete = 1;
              }

              sprintf(destfile, "%s\\winafsk.db", destDirName);
              VerifiedDeleteFile(hDlg, destfile);

              if(processcomplete)
              {                  
                CoInitialize(NULL);
        
                strcat(szStartMenuDir, WinAFSKdir);
                mkdir(szStartMenuDir);
        
                strcpy(sourcefile, szHomeDir);
                strcat(sourcefile, "\\WinAFSK.EXE");
                strcpy(destfile, szStartMenuDir);
                strcat(destfile, "\\WinAFSK.LNK");
                CreateLink(sourcefile, destfile, "WinAFSK");
        
                strcpy(sourcefile, szHomeDir);
                strcat(sourcefile, "\\README.TXT");
                strcpy(destfile, szStartMenuDir);
                strcat(destfile, "\\Readme.LNK");
                CreateLink(sourcefile, destfile, "WinAFSK Readme");
              }
              dlgNumber = dlgCOMPLETE;
              SendMessage(hDlg, WM_INITDIALOG, 0, 0);
              break;
          }
          break;

        case dlgUNINSTALL:
          if(LOWORD(wParam) != IDC_BUTTON1)
          {
            if(MessageBox(GetFocus(), pszUninstallCancelMsg, szAppName,
                         MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION) == 6)
            {
              PostQuitMessage(0);
              return FALSE;
            }
          }
          else
          {
            hWndTmp = FindWindow(szApplicationClassName, NULL);
            if(hWndTmp != NULL)
            {
//              SendMessage(hWndTmp, WM_DESTROY, 0, 0);
                SendMessage(hWndTmp, WM_CLOSE, 0, 0);
              Sleep(500);

              hWndTmp = FindWindow(szApplicationClassName, NULL);
              if(hWndTmp != NULL)
              {

                wsprintf(szBuffer, "%s\\WinAFSK.EXE", szHomeDir);
            
                if(bUnInstallFlag)
                  wsprintf(szBuffer, pszExeIsRunningMsg, "un");
                else
                  wsprintf(szBuffer, pszExeIsRunningMsg, "");
            
                MessageBox(NULL, szBuffer, szAppName, MB_OK);
            
                if (IsIconic(hWndTmp))
                  ShowWindow(hWndTmp, SW_RESTORE);
            
                SetForegroundWindow(hWndTmp);
            
                PostQuitMessage(0);
                return FALSE;
              }
            }
          
            
            //
            //  display the progress dialog
            //
            dlgNumber = dlgPROGRESS;
            SendMessage(hDlg,WM_INITDIALOG,0,0);
          
            hWndProgressBar = GetDlgItem(hDlg, IDC_PROGRESS1);
            hwndPT = GetDlgItem(hDlg, IDC_WELCOMEMSG);
            Display(hDlg, IDC_BUTTON1, SW_HIDE);
            SendMessage(hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0, iFileCount));
            SendMessage(hWndProgressBar, PBM_SETSTEP, (WPARAM) 1, 0);
            SendMessage(hWndProgressBar, PBM_STEPIT, 0, 0);
            ShowWindow(hDlg, SW_NORMAL);
            EnableWindow(mainhWnd, FALSE);
          
            SendMessage(hWndProgressBar, PBM_STEPIT, 0, 0);
        
            processcomplete = 0;
            strcpy(destDirName, szHomeDir);
            sprintf(szBuffer, "%s\\UNINST.EXE", destDirName);
            lRet = VerifiedDeleteFile(hDlg, szBuffer);
          
            for(iFileCount=0; iFileCount<FILECOUNT; iFileCount++)
            {
              strcpy(ShortName, FileList[iFileCount]);
              if(strlen(ShortName)>20)
              {
                szBuffer[0] = szBuffer[1] = szBuffer[2] = '.';
                szBuffer[3] = 0;
                strcat(szBuffer, &ShortName[strlen(ShortName)-23]);
                strcpy(ShortName, szBuffer);
              }
              sprintf(szBuffer, "Tiedosto: %s                ", ShortName);
              SetWindowText(hwndPT, szBuffer);
      
              sprintf(destfile, "%s\\%s", destDirName, FileList[iFileCount]);
        
              VerifiedDeleteFile(hDlg, destfile);
      
              SendMessage(hWndProgressBar, PBM_STEPIT, 0, 0);
              processcomplete = 1;
            }

            //
            //  delete the uninstall program
            //
            sprintf(destfile, "%s\\uninstall.exe", destDirName);
            VerifiedDeleteFile(hDlg, destfile);

            //
            //  delete the database file
            //
            sprintf(destfile, "%s\\winafsk.db", destDirName);
            VerifiedDeleteFile(hDlg, destfile);

            //
            //  delete the start menu and desktop stuff
            //
            strcat(szStartMenuDir,WinAFSKdir);

            strcpy(szBuffer, szStartMenuDir);
            strcat(szBuffer, "\\WinAFSK.LNK");
            unlink(szBuffer);

            strcpy(szBuffer, szStartMenuDir);
            strcat(szBuffer, "\\README.LNK");
            unlink(szBuffer);

            strcpy(szBuffer, szStartMenuDir);
            strcat(szBuffer, "\\UnInstall WinAFSK.LNK");
            unlink(szBuffer);

            rmdir(szStartMenuDir);

            strcpy(szBuffer, szDesktopDir);
            strcat(szBuffer, "\\WinAFSK.LNK");
            unlink(szBuffer);

            strcpy(szBuffer, szHomeDir);
            strcat(szBuffer, "\\SRC\\SETUP");
            rmdir(szBuffer);
        
            strcpy(szBuffer, szHomeDir);
            strcat(szBuffer, "\\SRC\\DEMOD");
            rmdir(szBuffer);
        
            strcpy(szBuffer, szHomeDir);
            strcat(szBuffer, "\\SRC");
            rmdir(szBuffer);
        
            DeleteRegistry();
            chdir(szWinDir);
            rmdir(destDirName);

            dlgNumber = dlgCOMPLETE;
            SendMessage(hDlg, WM_INITDIALOG, 0, 0);
          }
          break;

      }

    default:
      break;
  }
  return FALSE;
}

//
//   FUNCTION: CenterWindow(HWND, HWND)
//
//   PURPOSE: Centers one window over another.
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
//       This function will center one window over another ensuring that
//    the placement of the window is within the 'working area', meaning
//    that it is both within the display limits of the screen, and not
//    obscured by the tray or other framing elements of the desktop.
//
BOOL CenterWindow (HWND hwndChild, HWND hwndParent, int addx, int addy)
{
  RECT    rChild, rParent, rWorkArea;
  int     wChild, hChild, wParent, hParent;
  int     xNew, yNew;
  BOOL  bResult;

  // Get the Height and Width of the child window
  GetWindowRect (hwndChild, &rChild);
  wChild = rChild.right - rChild.left + addx;
  hChild = rChild.bottom - rChild.top + addy;

  // Get the Height and Width of the parent window
  //
  // If the parent hWnd is null, center the window on the screen
  //
  if(hwndParent)
  {
    GetWindowRect (hwndParent, &rParent);
    wParent = rParent.right - rParent.left;
    hParent = rParent.bottom - rParent.top;
  }
  else
  {
    rParent.left = rParent.top = 0;
    wParent = rParent.right  = GetSystemMetrics(SM_CXSCREEN);
    hParent = rParent.bottom = GetSystemMetrics(SM_CYSCREEN);
  }

  // Get the limits of the 'workarea'
  bResult = SystemParametersInfo(
     SPI_GETWORKAREA,  // system parameter to query or set
     sizeof(RECT),
     &rWorkArea,
     0);
  if (!bResult)
  {
     rWorkArea.left = rWorkArea.top = 0;
     rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
     rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
  }

  // Calculate new X position, then adjust for workarea
  xNew = rParent.left + ((wParent - wChild) /2);
  if (xNew < rWorkArea.left)
  {
     xNew = rWorkArea.left;
  } else if ((xNew+wChild) > rWorkArea.right)
  {
     xNew = rWorkArea.right - wChild;
  }

  // Calculate new Y position, then adjust for workarea
  yNew = rParent.top  + ((hParent - hChild) /2);
  if (yNew < rWorkArea.top)
  {
     yNew = rWorkArea.top;
  } else if ((yNew+hChild) > rWorkArea.bottom)
  {
     yNew = rWorkArea.bottom - hChild;
  }

  // Set it, and return
  return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

//
//  Callback for the directory selection dialog box
//
LRESULT CALLBACK BrowseCB(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  char buf[MAX_BUFFER];
  LPITEMIDLIST lp;

  switch (message)
  {
    case BFFM_INITIALIZED:
      SetWindowText(hDlg,"Choose a directory to install WinAFSK into");
      ShowWindow (hDlg, SW_HIDE);
      CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER), 0, 0);
      hWndText = CreateWindowEx(  WS_EX_CLIENTEDGE,"EDIT","",
                        ES_LEFT | ES_AUTOHSCROLL | WS_BORDER |
                        WS_CHILD | WS_TABSTOP | WS_VISIBLE,
                        10, 20,  298, 24, hDlg, NULL, hInst, NULL);
      originalTextProc = (WNDPROC)SetWindowLong(hWndText, GWL_WNDPROC, (long)TextProc);

      if(hFont==NULL)
        hFont = CreateFont(14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "MS Sans Serif");

      SendMessage(hWndText, WM_SETFONT, (WPARAM)hFont, 0L);
      SendMessage(hWndText, WM_PAINT, 0L, 0L);
      EnsurePathName(dlgInstallDir);
      SetWindowText(hWndText, dlgInstallDir);
      SendMessage(hDlg, BFFM_ENABLEOK , 1L, 1L);
      ShowWindow (hDlg, SW_SHOW);
      SetFocus(hWndText);
      return TRUE;

    case BFFM_SELCHANGED:
      lp = (LPITEMIDLIST)wParam;
      SHGetPathFromIDList(lp, buf);
      SetWindowText(hWndText, buf);
      if(strlen(buf))
      {
        EnsurePathName(buf);
        strcpy(dlgInstallDir,buf);
        SetWindowText(hWndText, dlgInstallDir);
      }
      return TRUE;

    case BFFM_VALIDATEFAILED:
      SendMessage(hDlg, BFFM_ENABLEOK , 1L, 1L);
      return TRUE;

    default:
      break;
  }
  return FALSE;
}

LRESULT VerifiedCopyFile(HWND hWnd, char *source, char *destination, BOOL recurseflag)
{
  int result;
  char buf[MAX_BUFFER];

  // if the destination file already exists, this will ensure it is writable
  _chmod(destination, _S_IREAD | _S_IWRITE);

  result = FileCopy(hWnd, source, destination);

  if(bCopyCancelFlag)
    return FALSE;

  if(!result)
  {
    if(GetFileAttributes(source) == -1)
    {
      // the source file doesnt exist
      if(source[0] == '.' && source[1] == '\\')
        source += 2;

      sprintf(buf,"The file named %s is missing from the installation package.",strupr(source));

    }
    else
    {
      if(GetFileAttributes(destination) == -1)
      {
        sprintf(buf,"Cannot create file - %s.",strupr(destination));
      }
      else
      {
        sprintf(buf,"The file named %s is busy. Please close it, and try again.",strupr(destination));
      }
    }
    MessageBox (GetFocus(), buf, szAppName, MB_OK);
    DestroyWindow(hWnd);
  }
  return result;
}

LRESULT CreateDir( char *dirName )
{
  char buf[MAX_BUFFER], path[MAX_BUFFER], *cp;
                     
  strcpy(buf, dirName);
  path[0]=0;
  cp = strtok(buf,"\\");
  strcat(path,cp);
  while(cp)
  {
    if(GetFileAttributes(path) == -1)
    {
      if(mkdir(path))
      {
        sprintf(buf,"cannot create directory %s",path);
        MessageBox(GetFocus(), buf, szAppName, 0);
        return FALSE;
      }
    }
    cp = strtok(NULL,"\\");
    if(cp)
    {
      strcat(path,"\\");
      strcat(path,cp);
    }
  }
  return TRUE;
}

LRESULT APIENTRY TextProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{ 
  if((uMsg == WM_SETFOCUS) || (uMsg == WM_KILLFOCUS))
  {
    GetWindowText(hWndText, dlgInstallDir, MAX_BUFFER-1);
    EnsurePathName(dlgInstallDir);
    SetWindowText(hWndText, dlgInstallDir);
  }

  return CallWindowProc(originalTextProc, hwnd, uMsg, 
    wParam, lParam);
} 

LRESULT FileCopy(HWND hDialog, char *szInputFile, char *szOutputFile)
{
  FILE *iFP, *oFP;
  unsigned long iFileSize, iBlockSize, iPosition;
  MSG msg;
  char *buf;

  // allocate a 1M buffer
#define FileCopy_BufferSize 0x100000

  // delete the target file, if one exists. checking the return
  // code of this call is not necessary
  unlink(szOutputFile);

  // open input and output files
  iFP = fopen(szInputFile,"rb");
  if(iFP == NULL)
    return FALSE;

  oFP = fopen(szOutputFile,"wb");
  if(oFP == NULL)
  {
    fclose(iFP);
    return FALSE;
  }

  // allocate buffer
  buf = malloc(FileCopy_BufferSize);
  if(buf == NULL)
  {
    fclose(iFP);
    fclose(oFP);
    return FALSE;
  }

  fseek(iFP,0L,SEEK_END);
  iFileSize = ftell(iFP);
  fseek(iFP,0L,SEEK_SET);
  iPosition = 0;
  while(iPosition<iFileSize && bCopyCancelFlag == FALSE)
  {
    iBlockSize = iFileSize - iPosition;
    if(iBlockSize > FileCopy_BufferSize)
    {
      iBlockSize = FileCopy_BufferSize;
    }
    iPosition += iBlockSize;
    fread(buf, iBlockSize, 1, iFP);
    fwrite(buf, iBlockSize, 1, oFP);
    iBytesCopied += iBlockSize;
    while(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
      DispatchMessage(&msg);
  }

  // close buffers and files
  free(buf);
  fclose(iFP);
  fclose(oFP);
  if(bCopyCancelFlag)
  {
    unlink(szOutputFile);
    return FALSE;
  }
  return TRUE;
}

BOOL RegisterProgram(char *szProgDir)
{
  HKEY hKey;
  long lRet;
  char buf[MAX_BUFFER];

  lRet = RegCreateKey(HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\UnInstall\\WinAFSK",&hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return FALSE;

  strcpy(buf,"WinAFSK");
  lRet = RegSetValueEx(hKey,"DisplayName",0,REG_SZ,buf,strlen(buf));
  if((LONG)ERROR_SUCCESS!=lRet)
    return FALSE;

  sprintf(buf,"%s\\UNINSTALL.EXE UNINSTALL",szProgDir);
  lRet = RegSetValueEx(hKey,"UninstallString",0,REG_SZ,buf,strlen(buf));
  if((LONG)ERROR_SUCCESS!=lRet)
     return FALSE;

  RegCloseKey(hKey);

  lRet = RegCreateKey(HKEY_LOCAL_MACHINE, "Software\\WinAFSK",&hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return FALSE;

  lRet = RegSetValueEx(hKey,"HomeDirectory",0,REG_SZ,szHomeDir,strlen(szHomeDir));
  if((LONG)ERROR_SUCCESS!=lRet)
    return FALSE;

  RegCloseKey(hKey);

  return TRUE;
}

void DeleteRegistry(void)
{
   long lRet;
   char *szKeyRef = "Software\\Microsoft\\Windows\\CurrentVersion\\UnInstall\\WinAFSK";
   lRet = RegDeleteKey(HKEY_LOCAL_MACHINE, szKeyRef);
   if((LONG)ERROR_SUCCESS!=lRet)
     return;
}

LRESULT VerifiedDeleteFile(HWND hWnd, char *destination)
{
  int result;
  char buf[MAX_BUFFER];
  MSG msg;

  while(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
    DispatchMessage(&msg);

  // if the destination file already exists, this will ensure it is writable
  _chmod(destination, _S_IREAD | _S_IWRITE);

  result = unlink(destination);
  if(result == 0)
    result = 1;

  if(!result)
  {
    if(GetFileAttributes(destination) == -1)
    {
      sprintf(buf,"Sharing violation - cannot delete %s. Try closing the program that is using it, and run the uninstall program again.",strupr(destination));
    }
    MessageBox (GetFocus(), buf, szAppName, MB_OK);
    DestroyWindow(hWnd);
  }
  return result;
}

void ReadRegistry(void)
{
  DWORD dwType, dwLen;
  long lRet, lType;
  HKEY hKey;

  szHomeDir[0]=0;

  lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Software\\WinAFSK", 0, KEY_QUERY_VALUE, &hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  dwLen = MAX_BUFFER-1;
  lRet = RegQueryValueEx(hKey,"HomeDirectory",0,&lType,szHomeDir,&dwLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  RegCloseKey(hKey);

  return;
}

void EnsurePathName(char *PathAndFile)
{
  int i, len, pos = 0;

  RemoveDoubleBackslash(PathAndFile);

  len = strlen(PathAndFile);

  if(len>0)
  {
    while(PathAndFile[len-1]=='\\')
    {
      PathAndFile[len-1] = 0;
      if(--len == 0)
        break;
    }
  }

  if(len>0)
  {
    for(i=pos=0;i<len;i++)
      if(PathAndFile[i] == '\\')
        pos = i+1;
  }

  if(stricmp(PathAndFile + pos,"WinAFSK")!=0)
    strcat(PathAndFile,"\\WinAFSK");
}

void GetSystemDirectories(char *ProgramFilesDir, char *StartMenuDir)
{
  DWORD dwType, dwLen;
  long lRet, lType;
  HKEY hKey;

  ProgramFilesDir[0]=0;

  lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Software\\Microsoft\\Windows\\CurrentVersion", 0, KEY_QUERY_VALUE, &hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  dwLen = MAX_BUFFER-1;
  lRet = RegQueryValueEx(hKey,"ProgramFilesDir",0,&lType,ProgramFilesDir,&dwLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  RegCloseKey(hKey);

  GetTempPath(MAX_BUFFER,szTmpDir);

  StartMenuDir[0]=0;
  lRet = RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 0, KEY_QUERY_VALUE, &hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  dwLen = MAX_BUFFER-1;
  lRet = RegQueryValueEx(hKey,"Programs",0,&lType,StartMenuDir,&dwLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  dwLen = MAX_BUFFER-1;
  lRet = RegQueryValueEx(hKey,"Desktop",0,&lType,szDesktopDir,&dwLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  RegCloseKey(hKey);

  return;
}

// CreateLink - uses the shell's IShellLink and IPersistFile interfaces
//   to create and store a shortcut to the specified object.
// Returns the result of calling the member functions of the interfaces.
// lpszPathObj - address of a buffer containing the path of the object
// lpszPathLink - address of a buffer containing the path where the
//   shell link is to be stored
// lpszDesc - address of a buffer containing the description of the
//   shell link

HRESULT CreateLink(LPCSTR lpszPathObj, LPSTR lpszPathLink, LPSTR lpszDesc)
{
  // assumes CoInitialize(NULL); has already been called

  HRESULT hres;
  IShellLink* psl;

  // Get a pointer to the IShellLink interface.
  hres = CoCreateInstance(&CLSID_ShellLink, NULL,
      CLSCTX_INPROC_SERVER, &IID_IShellLink, &psl);
  if (SUCCEEDED(hres))
  {
    IPersistFile* ppf;

    // Set the path to the shortcut target, and add the
    // description.
    psl->lpVtbl->SetPath(psl, lpszPathObj);
    psl->lpVtbl->SetDescription(psl, lpszDesc);

    // Query IShellLink for the IPersistFile interface for saving the
    // shortcut in persistent storage.
    hres = psl->lpVtbl->QueryInterface(psl, &IID_IPersistFile, &ppf);

    if (SUCCEEDED(hres))
    {
      WORD wsz[MAX_PATH];

      // Ensure that the string is ANSI.
      MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1,
          wsz, MAX_PATH);

      // Save the link by calling IPersistFile::Save.
      hres = ppf->lpVtbl->Save(ppf, wsz, TRUE);
      ppf->lpVtbl->Release(ppf);
    }
    psl->lpVtbl->Release(psl);
  }
  return hres;
}   

int CheckFileSpace(char *target)
{
  __int64 i64;
  char buf[MAX_BUFFER];
  char cDriveLetter;
  DWORD dwDiskInfo[4];
  
  //
  // check available diskspace in windows directory
  //
  buf[0] = cDriveLetter = szWinDir[0];
  buf[1] = szWinDir[1];
  buf[2] = szWinDir[2];
  buf[3] = 0;
  
  GetDiskFreeSpace(buf,  &dwDiskInfo[0],
                         &dwDiskInfo[1],
                         &dwDiskInfo[2],
                         &dwDiskInfo[3]);
  
  i64 = (__int64)dwDiskInfo[0]*dwDiskInfo[1]*dwDiskInfo[2];

  if(i64 < DiskSpaceRequired[0])
  {
    sprintf(buf,"Drive %c does not have enough free space available",toupper(cDriveLetter));
    MessageBox(GetFocus(),buf, szAppName,MB_OK);
    return FALSE;
  }
  
  //
  // check available diskspace in install directory
  //
  buf[0] = cDriveLetter = target[0];
  buf[1] = target[1];
  buf[2] = target[2];
  buf[3] = 0;
  GetDiskFreeSpace(buf, &dwDiskInfo[0],
                        &dwDiskInfo[1],
                        &dwDiskInfo[2],
                        &dwDiskInfo[3]);
  
  i64 = (__int64)dwDiskInfo[0]*dwDiskInfo[1]*dwDiskInfo[2];
  
  if(i64 < DiskSpaceRequired[1])
  {
    sprintf(buf,"Drive %c does not have enough free space available",toupper(cDriveLetter));
    MessageBox(GetFocus(),buf, szAppName,MB_OK);
    return FALSE;
  }

  return TRUE;
}

void ConvertSlashes(char *buf)
{
  while(*buf)
  {
    if(*buf == '/')
      *buf = '\\';
    buf++;
  }
}

void RemoveDoubleBackslash(char *buf)
{
  char local[MAX_BUFFER], *cp, *cpp;

  ConvertSlashes(buf);

  cpp = buf;
  cp = local;

  //
  //  this allows a \\ at the beginning of the path, for working across
  //  a network
  //
  *cp = *buf;
  cp++;
  buf++;

  //
  //  remove all double \'s after the first character
  //
  for(; *buf; buf++, cp++)
  {
    if((*cp = *buf) == '\\')
      while(*(buf+1) == '\\')
        buf++;
  }

  if(cp > (local+1))
    if(*(cp-1) == '\\')
      cp--;

  *cp = 0;
  strcpy(cpp,local);
}

void QuitInstall(void)
{
  //
  //  Deletes the files that are created in the TEMP file by the self
  //  extraction program (winzip)
  //
  int i;
  char szBuf[MAX_BUFFER];

  for(i=0;i<FILECOUNT;i++)
  {
    sprintf(szBuf,"%s%s",szTmpDir,FileList[i]);
    unlink(szBuf);
  }

  strcpy(szBuf,szTmpDir);
  strcat(szBuf,"SRC\\SETUP");
  rmdir(szBuf);

  strcpy(szBuf,szTmpDir);
  strcat(szBuf,"SRC");
  rmdir(szBuf);

  PostQuitMessage(0);
}

//
//  Checks that all the files in the install package are present
//
BOOL CheckForInstallFiles(void)
{
  FILE *fp;
  int i;

  for(i=0;i<FILECOUNT;i++)
    if((fp = fopen(FileList[i],"r"))==NULL)
    {
      wsprintf(szBuffer, "File %s is missing from the installation package. The installation cannot be performed.", FileList[i]);
      MessageBox(mainhWnd, szBuffer, szAppName, MB_OK);
      return FALSE;
    }
    else
      fclose(fp);

  return TRUE;
}

