/*
 *
 *      winafdmd.c v2.01b By Mikko Antero Gustafsson
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <windows.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <memory.h>
#include "winafsk.h"
#include "demod\demod.h"

struct demod_state hapn48_state;
struct demod_state afsk24_2_state;
struct demod_state afsk24_state;
struct demod_state afsk12_state;
struct demod_state fsk96_state;

struct
{
  unsigned char rxbuf[PACKET_LENGTH];  // usually 512 bytes
  unsigned char *rxptr;
  unsigned int rxstate;
  unsigned int rxbitstream;
  unsigned int rxbitbuf;
} receive_buffer;

void InitialiseDemodulators(void)
{
  afsk12_init(&afsk12_state);
  afsk24_init(&afsk24_state);
  afsk24_2_init(&afsk24_2_state);
  fsk96_init(&fsk96_state);
  hapn48_init(&hapn48_state);
}

//
//  Demodulate():  Demodulates the audio data contained in an input buffer.
//
void Demodulate(signed short *samplebuffer, int samplecount, int samplelength)
{
  float *buffer, *fptr;
  signed short *sptr;
  int bufferlength;

  buffer = (float*) malloc(sizeof(float)*samplecount*samplelength);

  bufferlength = samplecount*samplelength;
  for(fptr=buffer,sptr=samplebuffer;sptr-samplebuffer<bufferlength;fptr++,sptr++)
    *fptr = (*sptr) * (1.0/32768.0);

  afsk12_demod(&afsk12_state, buffer, bufferlength);
  afsk24_demod(&afsk24_state, buffer, bufferlength);
  afsk24_2_demod(&afsk24_2_state, buffer, bufferlength);
  fsk96_demod(&fsk96_state, buffer, bufferlength);
  hapn48_demod(&hapn48_state, buffer, bufferlength);

  free(buffer);
}

//
//  ReceivePacket(): Coordinates the process for receiving a packet and
//                   processing it as necessary
//
void ReceivePacket(struct demod_state *s, unsigned char *bp, unsigned int len)
{
  int lineCount;
  Record pr;
  char szBuf[64];
  char *cp;

  //
  //  disable edit control's redraw property
  //
  SendMessage( g_hLogTextWindow, WM_SETREDRAW, 0, 0 );

  //
  //  output the contents of the packet to the edit control
  //
  ParsePacket( s, bp, len, &pr );

  strcpy(pr.Modulation, s->dem_par->name);

  //
  //  Erase the space at the front of the message, that the parse
  //  routine puts in sometimes
  //
  if(pr.Message[0]==32)
    for(cp=pr.Message;*cp;cp++)
      *cp = *(cp+1);

  //
  //  Update the status information in the rebar control
  //
  UpdateRebarStatusBox(&pr);

  //
  //  add new item to the database list
  //
  if(g_hDatabaseWindow)
    AddDatabaseListItem(&pr);

  //
  //  add new record to the database
  //
  AddNewRecord(&pr);

  //
  //  re-enable edit control's redraw property
  //
  SendMessage(g_hLogTextWindow, WM_SETREDRAW, 1, 0 );

  //
  //  auto-scroll the edit control, if necessary
  //
  if( g_fAutoScroll )
  {
    lineCount = SendMessage( g_hLogTextWindow, EM_GETLINECOUNT, 0, 0 );
    SendMessage( g_hLogTextWindow, EM_LINESCROLL, 0, lineCount );
  }

  //
  //  trigger an update of the edit control
  //
  UpdateWindow( g_hLogTextWindow );

  wsprintf(szBuf, PacketCountMessage, ++g_iPacketcount);
  SendMessage(g_hStatusBar, SB_SETTEXT, 0, (LPARAM)&szBuf);

}

//
//  ParsePacket(): The function that converts the incoming binary
//                 packets, to a readable text format.
//
void ParsePacket(struct demod_state *s, unsigned char *bp, unsigned int len, Record *pr)
{
  unsigned char v1=1,cmd=0;
  unsigned char i,j;
  int charindex=0;
  char szBuf[RecordMessageMaxLength];
  char *pszTime;

  memset(pr,0,sizeof(Record));

  WinAFSK_printf(g_hLogTextWindow, "\r\n");

  WinAFSK_printf(g_hLogTextWindow, "%s: ", s->dem_par->name);

  time( &pr->TimeStamp );
  pszTime = ctime(&pr->TimeStamp);
  *(pszTime + strlen(pszTime) - 1) = 0;
  WinAFSK_printf(g_hLogTextWindow,"%s ", pszTime);
  free(pszTime);

	len -= 2;
  if (bp[1] & 1)
  {
    /*
     * FlexNet Header Compression
     */
    v1 = 0;
    cmd = (bp[1] & 2) != 0;

    strcpy(pr->From,"unknown");
    WinAFSK_printf(g_hLogTextWindow, "<%s> to ", pr->From);

    charindex = 0;

    i = (bp[2] >> 2) & 0x3f;
    if (i)
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }

    i = ((bp[2] << 4) | ((bp[3] >> 4) & 0xf)) & 0x3f;
    if (i) 
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }
  
    i = ((bp[3] << 2) | ((bp[4] >> 6) & 3)) & 0x3f;
    if (i) 
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }
  
    i = bp[4] & 0x3f;
    if (i) 
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }
  
    i = (bp[5] >> 2) & 0x3f;
    if (i) 
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }
  
    i = ((bp[5] << 4) | ((bp[6] >> 4) & 0xf)) & 0x3f;
    if (i) 
    {
      i += 0x20;
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }

    szBuf[charindex] = 0;

    WinAFSK_printf(g_hLogTextWindow, "-%u QSO Nr %u", bp[6] & 0xf, (bp[0] << 6) | (bp[1] >> 2));
    wsprintf(pr->To, "%s-%u", szBuf, bp[6] & 0xf);
    wsprintf(pr->Message, "QSO Nr %u", (bp[0] << 6) | (bp[1] >> 2));
    bp += 7;
    len -= 7;
  }
  else
  {
    /*
     * normal header
     */
    if (len < 15) 
      return;

    if ((bp[6] & 0x80) != (bp[13] & 0x80))
    {
      v1 = 0;
      cmd = (bp[6] & 0x80);
    }

    for(i = 7, charindex = 0; i < 13; i++)
      if ((bp[i] &0xfe) != 0x40)
      {
        WinAFSK_printf(g_hLogTextWindow, "%c",bp[i] >> 1);
        szBuf[charindex++] = bp[i] >> 1;
      }
    szBuf[charindex]=0;
    WinAFSK_printf(g_hLogTextWindow, "-%u to ",(bp[13] >> 1) & 0xf);
    wsprintf(pr->From,"%s-%u",szBuf,(bp[13] >> 1) & 0xf);

    for(i = 0, charindex = 0; i < 6; i++)
      if ((bp[i] &0xfe) != 0x40)
      {
        WinAFSK_printf(g_hLogTextWindow, "%c",bp[i] >> 1);
        szBuf[charindex++] = bp[i] >> 1;
      }
    szBuf[charindex]=0;
    WinAFSK_printf(g_hLogTextWindow, "-%u",(bp[6] >> 1) & 0xf);
    wsprintf(pr->To,"%s-%u",szBuf,(bp[6] >> 1) & 0xf);
    bp += 14;
    len -= 14;
        
    if ((!(bp[-1] & 1)) && (len >= 7))
    {
      WinAFSK_printf(g_hLogTextWindow, " via ");
      strcpy(pr->Message,"via ");
    }

    while ((!(bp[-1] & 1)) && (len >= 7))
    {
      for(i = 0, charindex = 0; i < 6; i++) 
        if ((bp[i] &0xfe) != 0x40)
        {
          WinAFSK_printf(g_hLogTextWindow, "%c",bp[i] >> 1);
          szBuf[charindex++] = bp[i] >> 1;
        }
      szBuf[charindex] = 0;
      WinAFSK_printf(g_hLogTextWindow, "-%u",(bp[6] >> 1) & 0xf);
      strcat(pr->Message, szBuf);
      wsprintf(szBuf,"-%u",(bp[6] >> 1) & 0xf);
      strcat(pr->Message, szBuf);

      bp += 7;
      len -= 7;

      if ((!(bp[-1] & 1)) && (len >= 7))
      {
        WinAFSK_printf(g_hLogTextWindow, ",");
        strcat(pr->Message, ",");
      }
    }
  }

  if(!len)
    return;

  i = *bp++;
  len--;

  j = v1 ? ((i & 0x10) ? '!' : ' ') : 
    ((i & 0x10) ? (cmd ? '+' : '-') : (cmd ? '^' : 'v'));

  if (!(i & 1))
  {
    /*
     * Info frame
     */
    WinAFSK_printf(g_hLogTextWindow, " I%u%u%c",(i >> 5) & 7,(i >> 1) & 7,j);
    wsprintf(szBuf," I%u%u%c",(i >> 5) & 7,(i >> 1) & 7,j);
    strcat(pr->Message,szBuf);
  }
  else if (i & 2)
  {
    /*
     * U frame
     */
    switch (i & (~0x10))
    {
      case 0x03:
        WinAFSK_printf(g_hLogTextWindow, " UI%c",j);
        wsprintf(szBuf," UI%c",j);
        strcat(pr->Message,szBuf);
        break;
      case 0x2f:
        WinAFSK_printf(g_hLogTextWindow, " SABM%c",j);
        wsprintf(szBuf," SABM%c",j);
        strcat(pr->Message,szBuf);
        break;
      case 0x43:
        WinAFSK_printf(g_hLogTextWindow, " DISC%c",j);
        wsprintf(szBuf," DISC%c",j);
        strcat(pr->Message,szBuf);
        break;
      case 0x0f:
        WinAFSK_printf(g_hLogTextWindow, " DM%c",j);
        wsprintf(szBuf," DM%c",j);
        strcat(pr->Message,szBuf);
        break;
      case 0x63:
        WinAFSK_printf(g_hLogTextWindow, " UA%c",j);
        wsprintf(szBuf," UA%c",j);
        strcat(pr->Message,szBuf);
        break;
      case 0x87:
        WinAFSK_printf(g_hLogTextWindow, " FRMR%c",j);
        wsprintf(szBuf," FRMR%c",j);
        strcat(pr->Message,szBuf);
        break;
      default:
        WinAFSK_printf(g_hLogTextWindow, " unknown U (0x%x)%c",i & (~0x10),j);
        wsprintf(szBuf," unknown U (0x%x)%c",i & (~0x10),j);
        strcat(pr->Message,szBuf);
        break;
    }
  }
  else
  {
    /*
     * supervisory
     */
    switch (i & 0xf)
    {
      case 0x1:
        WinAFSK_printf(g_hLogTextWindow, " RR%u%c",(i >> 5) & 7,j);
        wsprintf(szBuf," RR%u%c",(i >> 5) & 7,j);
        break;
      case 0x5:
        WinAFSK_printf(g_hLogTextWindow, " RNR%u%c",(i >> 5) & 7,j);
        wsprintf(szBuf," RNR%u%c",(i >> 5) & 7,j);
        break;
      case 0x9:
        WinAFSK_printf(g_hLogTextWindow, " REJ%u%c",(i >> 5) & 7,j);
        wsprintf(szBuf," REJ%u%c",(i >> 5) & 7,j);
        break;
      default:
        WinAFSK_printf(g_hLogTextWindow, " unknown S (0x%x)%u%c", i & 0xf, (i >> 5) & 7, j);
        wsprintf(szBuf," unknown S (0x%x)%u%c", i & 0xf, (i >> 5) & 7, j);
        break;
    }
    strcat(pr->Message,szBuf);
  }

  if (!len)
  {
    WinAFSK_printf(g_hLogTextWindow, "\r\n");
    return;
  }

  WinAFSK_printf(g_hLogTextWindow, " pid=%02X\r\n", *bp);
  wsprintf(szBuf," pid=%02X ", *bp++);
  strcat(pr->Message,szBuf);

  len--;
  j = 0;

  charindex = 0;
  while (len)
  {
    i = *bp++;
    if ((i >= 32) && (i < 128))
    {
      WinAFSK_printf(g_hLogTextWindow, "%c",i);
      szBuf[charindex++] = i;
    }
    else if (i == 13)
    {
      if (j)
      {
        WinAFSK_printf(g_hLogTextWindow, "\r\n");
        szBuf[charindex++] = ' ';
      }
      j = 0;
    }
    else
    {
      WinAFSK_printf(g_hLogTextWindow, ".");
      szBuf[charindex++] = '.';
    }
    if (i >= 32)
      j = 1;

    len--;
  }

  if (j)
    WinAFSK_printf(g_hLogTextWindow, "\r\n");

  szBuf[charindex] = 0;

  strcat(pr->Message,szBuf);

}
    
