/*
 *
 *      winafsk.c v2.01c By Mikko Antero Gustafsson
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <memory.h>
#include "winafsk.h"

char *g_szAppName  = "WinAFSK";
char *AboutMessage = "WinAFSK v2.01c By Mikko Antero Gustafsson\r\n\r\nThis program allows you to demodulate AFSK1200, AFSK2400, HAPN4800 and FSK9600 packet radio traffic,\r\nby sampling the audio from your default sound device.\r\n";
char *EnvironmentMessage = "\r\nWinAFSK is now running in a Windows %s environment.\r\nThe maximum text log length is %s.\r\n";
char *PacketCountMessage = "Packets received: %i";

HWND CreateRebar(HWND hWnd);
HWND CreateToolBar(HWND hWnd);
void OpenScopeWindow(void);
void OpenLogWindow(void);
void OpenDatabaseWindow(void);
void SwitchAudioDevice(void);
void GetAudioDeviceID();

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK MDIChildWndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);

//
//  MDI child window titles and class names
//
char g_szDatabaseTitle[] = "Database";
char g_szDatabaseClass[] = "WinAFSKDatabaseClass";
char g_szLogTitle[]      = "Log";
char g_szLogClass[]      = "WinAFSKLogClass";
char g_szScopeTitle[]    = "Scope";
char g_szScopeClass[]    = "WinAFSKScopeClass";
char g_szVolumeTitle[]    = "Volume";
char g_szVolumeClass[]    = "WinAFSKVolumeClass";

//
//  Database related global variables
//
char g_szDatabaseFileName[] = "winafsk.db";
FILE *g_fpDB;

//
//  Global handles 
//
HINSTANCE g_hInst            = NULL;
HWND g_hMDIClient            = NULL;
HWND g_hStatusBar            = NULL;
HWND g_hRebar                = NULL;
HWND g_hMainWindow           = NULL;
HWND g_hLogTextWindow        = NULL;
HWND g_hLogWindow            = NULL;
HWND g_hScopeWindow          = NULL;
HWND g_hVolumeWindow         = NULL;
HWND g_hMicrophoneVolumeTrackBar = NULL;
HWND g_hOutputVolumeTrackBar = NULL;
HWND g_hMicrophoneVolumeLabel = NULL;
HWND g_hOutputVolumeLabel    = NULL;
HWND g_hAudioDeviceLabel     = NULL;
HWND g_hAudioDeviceComboBox  = NULL;
HWND g_hDatabaseWindow       = NULL;
HWND g_hDatabaseListBox      = NULL;
HWND g_hDatabaseTextBox      = NULL;
HWND g_hRebarStatusText      = NULL;
HFONT g_hFont                = NULL;
HMENU g_hMenu                = NULL;
HICON g_hSmallIcon           = NULL;
BOOL  g_iPacketcount         = 0;
BOOL  fMDIClientArrange      = TRUE;
BOOL  g_fMainWindowSizing    = FALSE;
HBRUSH g_hScopeBackgroundBrush = NULL;
HCURSOR g_hCursor            = NULL;
int g_iAudioDeviceID         = 0;

HMIXER g_hMixer;
int g_iMicrophoneVolumeMixerControlID = NULL;
int g_iOutputVolumeMixerControlID = NULL;
char g_AudioDeviceIdentifier[1024];

HFONT hLabelFont = NULL;

//
//  Variables used for debugging 
//
char szDebugBuf[1024];
BOOL bDebug                  = FALSE;

//
//  MDI window creation messages
//
#define MDI_CREATESCOPEWINDOW     1000
#define MDI_CREATELOGWINDOW       1001
#define MDI_CREATEDATABASEWINDOW  1002
#define MDI_CREATEVOLUMEWINDOW    1003

//
//  Maximum string length in bytes
//
#define EDIT_LENGTH 16384

char bufText[EDIT_LENGTH];

//
//  The number of audio buffers to use. Two is enough, unless
//  the AUDIO_BUFFER_SIZE is set really low
//
#define AUDIO_BUFFER_SIZE   (CORRLEN*1200*sizeof(short))
#define NUM_AUDIO_BUFFERS   2

//
//  Toolbar values and variables
//
#define TOOLBARWIDTH       310 
#define TOOLBARBUTTONWIDTH 40
#define TOOLBARHEIGHT      40

TBBUTTON tbb[] = {
  {0, IDM_SCOPEBUTTON, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, NULL},
  {1, IDM_LOGBUTTON, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, NULL},
  {2, IDM_DBBUTTON, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, NULL},
  {3, IDM_VOLUMEBUTTON, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, NULL},
  {0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0L, 0},
  {0, 0, TBSTATE_ENABLED, TBSTYLE_SEP, 0L, 0},
  {4, IDM_ARRANGEBUTTON, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, NULL}
  };

//
//  Width values for the columns in the Database window list
//
int g_iHeaderWidths[] = { 80, 120, 80, 80, 512};

//
//  Scope Window related variables and definitions
//
#define IDSCOPETIMER       1
#define SCOPEFRAMELENGTH   ((AUDIO_BUFFER_SIZE/sizeof(short)*50)/FREQ_SAMP)
#define SCOPEFRAMEMAX      20
#define SCOPEMAXWIDTH      306
#define SCOPEMAXHEIGHT     231
#define SCOPE_NOT_READY_YET -1
int     g_hScopeFrameCount   = 0;
POINT   newScopePoints[AUDIO_BUFFER_SIZE/sizeof(short)];
POINT   oldScopePoints[AUDIO_BUFFER_SIZE/sizeof(short)];

//
// Macros for determining operating system version
//
#if defined (WIN32)
   #define IS_WIN32 TRUE
#else
   #define IS_WIN32 FALSE
#endif
#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

//
//  Audio device structures
//
typedef struct _LevelStreamData
{
  LPWAVEHDR alpWave[NUM_AUDIO_BUFFERS]; 
  PCMWAVEFORMAT *pwf; 
  HWAVEIN hwav; 
  int buffersize; 
} LEVELSTREAMDATA, *PLEVELSTREAMDATA;

//
//  Miscellaneous global variables
//
char                g_fIncomingPacket;
signed short       *pAudioBuffers[NUM_AUDIO_BUFFERS];
unsigned short     *g_pScopeBuffer;
int                 g_iScopeIndex=SCOPE_NOT_READY_YET;
PLEVELSTREAMDATA    pInfo; 
PCMWAVEFORMAT       waveformat; 
BOOL                g_fAutoScroll =  FALSE;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpszCmdParam, int nCmdShow)
{
  MSG  Msg;
  WNDCLASSEX WndClassEx;
  HWND hWnd;

  //
  //  Check if the program is already running, and set focus to it and exit.
  //
  if(hWnd=FindWindow(g_szAppName,NULL))
  {
    SetWindowPos(hWnd, HWND_TOP, 0, 0, 0, 0,
                 SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
    SetForegroundWindow(hWnd);
    return TRUE;
  }

#define PATHLENGTH 1024
  char szBuffer[2][PATHLENGTH];

  ReadRegistry(PATHLENGTH, (unsigned char *)&szBuffer[0][0]);

  if(!SetCurrentDirectory((const char *)&szBuffer[0][0]))
  {
    wsprintf(&szBuffer[1][0],"Unable to access registered directory %s. Please reinstall WinAFSK.",&szBuffer[0]);
    MessageBox(NULL,&szBuffer[1][0], g_szAppName,MB_OK);
    return FALSE;
  }

  SetPriorityClass(hInstance, REALTIME_PRIORITY_CLASS);

  if(!OpenDatabase())
  {
    MessageBox(GetFocus(), "Unable to open the database", g_szAppName, MB_ICONEXCLAMATION);
    return -1;
  }

  InitCommonControls();

  g_hInst = hInstance;
  
  WndClassEx.cbSize          = sizeof(WNDCLASSEX);
  WndClassEx.style           = CS_HREDRAW | CS_VREDRAW;
  WndClassEx.lpfnWndProc     = WndProc;
  WndClassEx.cbClsExtra      = 0;
  WndClassEx.cbWndExtra      = 0;
  WndClassEx.hInstance       = hInstance;
  WndClassEx.hIcon           = LoadIcon(g_hInst, MAKEINTRESOURCE(IDC_ICONBIG));

  //
  //  The instance argument in LoadCursor must be NULL
  //
  WndClassEx.hCursor         = g_hCursor = LoadCursor(NULL, IDC_ARROW);
  WndClassEx.hbrBackground   = (HBRUSH)(COLOR_3DSHADOW+1);
  WndClassEx.lpszMenuName    = "MAIN";
  WndClassEx.lpszClassName   = g_szAppName;
  WndClassEx.hIconSm         = g_hSmallIcon = LoadIcon(g_hInst, MAKEINTRESOURCE(IDC_ICONSMALL));

  if(!RegisterClassEx(&WndClassEx))
     return -1;
  
  WndClassEx.lpfnWndProc     = MDIChildWndProc;
  WndClassEx.lpszMenuName    = NULL;
  WndClassEx.lpszClassName   = g_szLogClass;
  WndClassEx.hbrBackground   = (HBRUSH)GetStockObject(WHITE_BRUSH);
  
  if(!RegisterClassEx(&WndClassEx))
     return -1;

  WndClassEx.lpfnWndProc     = MDIChildWndProc;
  WndClassEx.lpszMenuName    = NULL;
  WndClassEx.lpszClassName   = g_szVolumeClass;
  WndClassEx.hbrBackground   = CreateSolidBrush(RGB(0xEC,0xE9,0xD8));
  
  if(!RegisterClassEx(&WndClassEx))
     return -1;
  
  WndClassEx.lpfnWndProc     = MDIChildWndProc;
  WndClassEx.lpszMenuName    = NULL;
  WndClassEx.lpszClassName   = g_szScopeClass;
  WndClassEx.hbrBackground   = g_hScopeBackgroundBrush = CreateSolidBrush(RGB(16,96,16));
  
  if(!RegisterClassEx(&WndClassEx))
     return -1;

  WndClassEx.lpfnWndProc     = MDIChildWndProc;
  WndClassEx.lpszMenuName    = NULL;
  WndClassEx.lpszClassName   = g_szDatabaseClass;
  WndClassEx.hbrBackground   = (HBRUSH)(COLOR_3DSHADOW+1);
  
  if(!RegisterClassEx(&WndClassEx))
     return -1;

  g_hMainWindow = CreateWindowEx(NULL, g_szAppName,
     g_szAppName, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
     NULL, NULL, hInstance, NULL);
  
  if (g_hMainWindow == NULL)
     return -1;
  
  CenterWindow(g_hMainWindow, NULL, 0, 0);
  ShowWindow(g_hMainWindow, nCmdShow);
  UpdateWindow(g_hMainWindow);

  //
  //  initialise the audio input
  //
  waveformat.wf.nChannels = 1;
  waveformat.wBitsPerSample = BITS_PER_SAMP; 
  waveformat.wf.nSamplesPerSec = FREQ_SAMP;

  if(!OpenStream(&waveformat))
  {
    MessageBox(g_hMainWindow, "Unable to open the default audio device. Try closing the program that is using it, and then run WinAFSK again.", g_szAppName, MB_ICONEXCLAMATION);
    return 1;
  }

  InitialiseDemodulators();

  OpenMixer();

  MixerChange(g_iMicrophoneVolumeMixerControlID);
  MixerChange(g_iOutputVolumeMixerControlID);

  while(GetMessage(&Msg, NULL, 0, 0))
  {
     if (!TranslateMDISysAccel(g_hMDIClient, &Msg))
     {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
     }
  }
  return Msg.wParam;
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
  int id, no, i;
  INITCOMMONCONTROLSEX ic;
  REBARBANDINFO rbBand;
  char szStr[256], szStr2[128], szStr3[128];
  RECT rc;
  CLIENTCREATESTRUCT ccs;
  int iStatusWidths[] = {200, -1};
  RECT rectClient, rectStatus, rectTool;
  UINT uToolHeight, uStatusHeight, uClientAlreaHeight;
  LPNMHDR lpnmhdr;
  int RebarY=0, ClientX=0, ClientY=0;
  HWND hChild, hChild2;
  float fx, fy;

  switch(Message)
  {
    case MM_MIXM_CONTROL_CHANGE:
      //MessageBox(GetFocus(),"MM_MIXM_CONTROL_CHANGE","",MB_OK);
      MixerChange((unsigned long)lParam);
      break;

    case WM_ENTERSIZEMOVE:
      g_fMainWindowSizing = TRUE;
      break;

    case WM_EXITSIZEMOVE:
      g_fMainWindowSizing = FALSE;
      break;

    case MM_WIM_DATA:
      g_iScopeIndex = SCOPE_NOT_READY_YET;
      i = StreamData((HWAVEIN)wParam, (LPWAVEHDR)lParam);
      return(TRUE);

    case WM_NOTIFY:
      if(wParam == ID_REBAR)
      {
        lpnmhdr = (LPNMHDR)lParam;
        if(lpnmhdr->code == RBN_HEIGHTCHANGE)
        {
          GetWindowRect(g_hRebar, &rc);
          RebarY = rc.bottom - rc.top;
          
          GetWindowRect(g_hMainWindow, &rc);
          ClientX = rc.right - rc.left;
          ClientY = rc.bottom - rc.top - RebarY;
          
          GetWindowRect(g_hStatusBar, &rc);
          ClientY -= rc.bottom - rc.top;
          
          ClientY -= GetSystemMetrics(SM_CYMENUSIZE);
          ClientY -= GetSystemMetrics(SM_CYCAPTION);
          
          id = GetSystemMetrics(SM_CYSIZEFRAME)*2;
          ClientY -= id;
          ClientX -= id;
          
          MoveWindow(g_hMDIClient, 0, RebarY, ClientX, ClientY, TRUE);
        }
      }
      else
        return (DefWindowProc(hwnd, Message, wParam, lParam));
      break;

    case WM_CREATE:
      //
      // Find window menu where children will be listed
      //
      ccs.hWindowMenu  = GetSubMenu(g_hMenu = GetMenu(hwnd), 1);
      ccs.idFirstChild = ID_MDI_FIRSTCHILD;
      g_hMDIClient = CreateWindowEx(WS_EX_CLIENTEDGE, "mdiclient", NULL,
           WS_CHILD | WS_CLIPCHILDREN | WS_VSCROLL | WS_HSCROLL,
           CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
           hwnd, (HMENU)ID_MDI_CLIENT, g_hInst, (LPVOID)&ccs);
      ShowWindow(g_hMDIClient, SW_SHOW);

      g_hStatusBar = CreateWindowEx(0, STATUSCLASSNAME, NULL,
           WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP, 0, 0, 0, 0,
           hwnd, (HMENU)ID_STATUSBAR, g_hInst, NULL);
      SendMessage(g_hStatusBar, SB_SETPARTS, 2, (LPARAM)iStatusWidths);

      wsprintf(szStr, PacketCountMessage, 0);
      SendMessage(g_hStatusBar, SB_SETTEXT, 0, (LPARAM)&szStr);

      ic.dwSize = sizeof(INITCOMMONCONTROLSEX);
      ic.dwICC = ICC_COOL_CLASSES;
      InitCommonControlsEx(&ic);
      g_hRebar = CreateRebar(hwnd);

      ZeroMemory(&rbBand, sizeof(REBARBANDINFO));
      rbBand.cbSize = sizeof(REBARBANDINFO);
      rbBand.fMask  = RBBIM_TEXT | RBBIM_STYLE |
                      RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_SIZE;
      rbBand.fStyle = RBBS_CHILDEDGE;
      rbBand.cxMinChild = 0;
      rbBand.cyMinChild = TOOLBARHEIGHT;

      hChild =  CreateToolBar(g_hRebar);

      rbBand.lpText = NULL;
      rbBand.hwndChild = hChild;
      GetWindowRect(g_hRebar, &rc);
      rbBand.cx = TOOLBARWIDTH;
      SendMessage(g_hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);

      g_hRebarStatusText = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", "",
                                 WS_CHILD | WS_VISIBLE | 
                                 ES_LEFT | ES_MULTILINE |
                                 ES_WANTRETURN | ES_READONLY,
                                 CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                                 CW_USEDEFAULT, g_hRebar, (HMENU)IDC_CHILD_EDIT,
                                 g_hInst, NULL);

      g_hFont = CreateFont(16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "Arial");

      if(g_hFont == NULL)
        return FALSE;

      SendMessage(g_hRebarStatusText, WM_SETFONT, (WPARAM)g_hFont, MAKELONG(TRUE, 0));

      rbBand.lpText = "";
      rbBand.hwndChild = g_hRebarStatusText;
      rbBand.cx = (rc.right - rc.left)-173;
      SendMessage(g_hRebar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);

      OpenLogWindow();
      OpenDatabaseWindow();

      hLabelFont = CreateFont(12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "MS Sans Serif");

      wsprintf(szStr,"Audio sample rate %iHz\r\n%i bits per sample.", FREQ_SAMP, BITS_PER_SAMP);
      SetWindowText(g_hRebarStatusText, szStr);

      WinAFSK_printf(g_hLogTextWindow, "\r\nIf no data is received, try adjusting the audio volume on your\r\nradio, or the level of the Microphone control in the Volume window.\r\nAlso, take a look at the \"Getting WinAFSK working\" page in the help section.\r\n");
               
      return 0;

    case WM_COMMAND:
    {
      switch(LOWORD(wParam))
      {
        case IDM_AUTOSCROLL:
          if(g_fAutoScroll)
          {
            CheckMenuItem (g_hMenu, IDM_AUTOSCROLL, MF_UNCHECKED);
            g_fAutoScroll = FALSE;
          }
          else
          {
            CheckMenuItem (g_hMenu, IDM_AUTOSCROLL, MF_CHECKED);
            g_fAutoScroll = TRUE;
          }
          return 0;

        case IDM_CONTENTS:
           ShellExecute(NULL,"open","winafsk.chm","",".",SW_SHOWNORMAL);
           return 0;

        case IDM_ABOUT:
           MessageBox (hwnd, AboutMessage,
                       g_szAppName, MB_ICONINFORMATION | MB_OK);
           return 0;

        case IDM_LOGBUTTON:
          if(g_hLogWindow==NULL)
            OpenLogWindow();
          else
          {
            ShowWindow(g_hLogWindow,SW_SHOWNORMAL);
            SetFocus(g_hLogWindow);
          }
          break;

        case IDM_VOLUMEBUTTON:
          if(g_hVolumeWindow==NULL)
            OpenVolumeWindow();
          else
          {
            ShowWindow(g_hVolumeWindow,SW_SHOWNORMAL);
            SetFocus(g_hVolumeWindow);
          }
          break;

        case IDM_SCOPEBUTTON:
          if(g_hScopeWindow==NULL)
            OpenScopeWindow();
          else
          {
            ShowWindow(g_hScopeWindow,SW_SHOWNORMAL);
            SetFocus(g_hScopeWindow);
          }
          break;

        case IDM_DBBUTTON:
          if(g_hDatabaseWindow==NULL)
            OpenDatabaseWindow();
          else
          {
            ShowWindow(g_hDatabaseWindow,SW_SHOWNORMAL);
            SetFocus(g_hDatabaseWindow);
          }
          break;

        case IDM_ARRANGEBUTTON:
          fMDIClientArrange = TRUE;
          SendMessage(g_hMainWindow,WM_SIZE,0,0);
          break;

        case IDM_EXIT:
          PostMessage(g_hMainWindow, WM_CLOSE, 0, 0);
          break;

        case CM_WINDOW_TILEHORZ:
           PostMessage(g_hMDIClient, WM_MDITILE, MDITILE_HORIZONTAL, 0);
           break;

        case CM_WINDOW_TILEVERT:
           PostMessage(g_hMDIClient, WM_MDITILE, MDITILE_VERTICAL, 0);
           break;

        case CM_WINDOW_CASCADE:
           PostMessage(g_hMDIClient, WM_MDICASCADE, 0, 0);
           break;

        case CM_WINDOW_ARRANGE:
           PostMessage(g_hMDIClient, WM_MDIICONARRANGE, 0, 0);
           break;

        default:
          {
            if(LOWORD(wParam) >= ID_MDI_FIRSTCHILD)
            {
              DefFrameProc(hwnd, g_hMDIClient, Message, wParam, lParam);
            }
            else
            {
              HWND hChild;
              hChild = (HWND)SendMessage(g_hMDIClient, WM_MDIGETACTIVE,0,0);
              if(hChild)
              {
                SendMessage(hChild, WM_COMMAND, wParam, lParam);
              }
            }
          }
        }
      }
      break;

    case WM_SIZE:
      if(hwnd == g_hMainWindow)
      {
        SendMessage(g_hRebar, WM_SIZE, 0, 0);
        SendMessage(g_hStatusBar, WM_SIZE, 0, 0);
        GetClientRect(hwnd, &rectClient);
        GetWindowRect(g_hStatusBar, &rectStatus);
        GetWindowRect(g_hRebar, &rectTool);
        uToolHeight = rectTool.bottom - rectTool.top;
        uStatusHeight = rectStatus.bottom - rectStatus.top;
        uClientAlreaHeight = rectClient.bottom;
        MoveWindow(g_hMDIClient, 0, uToolHeight, rectClient.right, uClientAlreaHeight - uStatusHeight - uToolHeight, TRUE);
        if(fMDIClientArrange)
        {
          GetClientRect(g_hMDIClient, &rc);
          fx = (float)(rc.right / 3);
          fy = (float)(rc.bottom / 2);
          if(!g_hScopeWindow)
            OpenScopeWindow();
          MoveWindow(g_hScopeWindow, 0, 0, (int)fx, (int)fy, TRUE);
          ShowWindow(g_hScopeWindow, SW_SHOWNORMAL);
          if(!g_hDatabaseWindow)
            OpenDatabaseWindow();
          MoveWindow(g_hDatabaseWindow, (int)fx, 0, rc.right - (int)fx, (int)fy, TRUE);
          ShowWindow(g_hDatabaseWindow, SW_SHOWNORMAL);
          if(!g_hVolumeWindow)
            OpenVolumeWindow();
          MoveWindow(g_hVolumeWindow, 0, (int)fy, (int)fx, rc.bottom - (int)fy, TRUE);
          ShowWindow(g_hVolumeWindow, SW_SHOWNORMAL);
          if(!g_hLogWindow)
            OpenLogWindow();
          MoveWindow(g_hLogWindow, (int)fx, (int)fy, rc.right - (int)fx, rc.bottom - (int)fy, TRUE);
          ShowWindow(g_hLogWindow, SW_SHOWNORMAL);
          fMDIClientArrange = FALSE;
        }
      }
      break;

    case WM_CLOSE:
      KillTimer(g_hScopeWindow,IDSCOPETIMER);
      CloseStream();
      CloseDatabase();
      mixerClose(g_hMixer);
      DestroyWindow(hwnd);
      break;

    case WM_DESTROY:
      PostQuitMessage(0);
      break;

    default:
      return DefFrameProc(hwnd, g_hMDIClient, Message, wParam, lParam);
  }
  return 0;
}

LRESULT CALLBACK MDIChildWndProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
  HWND        hWndChild;
  RECT        rc;
  HDC         hDC;
  HPEN        hPen, hOldPen;
  PAINTSTRUCT ps;
  int         i, ii, iWidth, iXIncrement, iYIncrement, iLevel, iBufIncrement;
  unsigned int ui;
  float       fWidth, fHeight;
  float       fWidthRatio, fHeightRatio;
  CREATESTRUCT *pCS;
  MDICREATESTRUCT *pMDICS;
  unsigned char *cp;
  signed short *sp;
  static BOOL fScopeIsPainting = FALSE;
  static BOOL fTimerIsPainting = FALSE;
  time_t t;

  switch(Message)
  {

    case WM_VSCROLL:
      if((HWND)lParam == g_hOutputVolumeTrackBar ||
         (HWND)lParam == g_hMicrophoneVolumeTrackBar)
      {
            ui = SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
            ui = ((unsigned long)100 - ui) << 1;
            ui *= 327;
            if((HWND)lParam == g_hMicrophoneVolumeTrackBar)
              SetVolume(ui, MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE);
            else
              SetVolume(ui, MIXERLINE_COMPONENTTYPE_DST_SPEAKERS);
      }
      return (DefWindowProc(hwnd, Message, wParam, lParam));

    case WM_KEYDOWN:
      if(wParam == VK_F1)
        ShellExecute(NULL,"open","winafsk.chm","",".",SW_SHOWNORMAL);
      break;

    case WM_CTLCOLORSTATIC:
      if(hwnd==g_hLogWindow)
      {
        SetBkColor((HDC)wParam, RGB(255,255,255));
        return (long)GetStockObject(WHITE_BRUSH);
      }
      break;

    case WM_SYSCOMMAND:
      //
      //  Prevents the scope window from being maximized
      //
      if(hwnd == g_hScopeWindow && wParam == SC_MAXIMIZE)
        return FALSE;
      break;

    case WM_DESTROY:
      if(hwnd==g_hScopeWindow)
      {
        KillTimer(g_hScopeWindow,IDSCOPETIMER);
        g_iScopeIndex = SCOPE_NOT_READY_YET;
        g_hScopeWindow = NULL;
      }
      else
      if(hwnd==g_hLogWindow)
      {
        g_hLogWindow = NULL;
        g_hLogTextWindow = NULL;
      }
      else
      if(hwnd==g_hDatabaseWindow)
      {
        g_hDatabaseWindow = NULL;
        g_hDatabaseListBox = NULL;
      }
      else
      if(hwnd==g_hVolumeWindow)
      {
        g_hVolumeWindow = NULL;
      }
      break;

    case WM_TIMER:
      if(hwnd == g_hScopeWindow)
      {
        if(g_hScopeFrameCount++ < SCOPEFRAMEMAX &&
           g_iScopeIndex != SCOPE_NOT_READY_YET &&
           !IsIconic(g_hScopeWindow) &&
           !fScopeIsPainting)
        {
          fTimerIsPainting = TRUE;
          GetClientRect(hwnd,&rc);
          InvalidateRect(hwnd,&rc,FALSE);
          UpdateWindow(hwnd);
          fTimerIsPainting = FALSE;
        }
        else
          KillTimer(g_hScopeWindow, IDSCOPETIMER);
      }
      break;

    case WM_PAINT:
      fScopeIsPainting = TRUE;
      hDC = BeginPaint(hwnd, &ps);
      if(hwnd == g_hScopeWindow &&
         g_iScopeIndex != SCOPE_NOT_READY_YET &&
         !IsIconic(g_hScopeWindow) &&
         !g_fMainWindowSizing)
      {
        GetClientRect(hwnd,&rc);

        if(fTimerIsPainting && g_iScopeIndex==0)
          FillRect(hDC,&rc,g_hScopeBackgroundBrush);

        fWidth = (float)(rc.right - rc.left);
        iBufIncrement = (AUDIO_BUFFER_SIZE/sizeof(short)/(1000/SCOPEFRAMELENGTH));
        fWidthRatio = fWidth / iBufIncrement;

        iWidth = (int)fWidth;

        fHeight = (float)(rc.bottom - rc.top);
        fHeightRatio = (float)(fHeight / 256.0);

        hPen = CreatePen(PS_SOLID, 1, RGB(32, 128, 32));
        hOldPen = (HPEN)SelectObject(hDC,hPen);
        iLevel = iWidth / 20;
        iXIncrement = (iWidth/2);
        MoveToEx(hDC, iXIncrement, 0, NULL);
        LineTo(hDC, iXIncrement, (int)(fHeightRatio * 256.0));
        iXIncrement -= (iLevel/2);

        for(i=80;i<=176;i+=32)
        {
          MoveToEx(hDC, 0, (int)(fHeightRatio * i), NULL);
          LineTo(hDC, iLevel, (int)(fHeightRatio * i));
          MoveToEx(hDC, iWidth - iLevel, (int)(fHeightRatio * i), NULL);
          LineTo(hDC, iWidth, (int)(fHeightRatio * i));
          MoveToEx(hDC, iXIncrement, (int)(fHeightRatio * i), NULL);
          LineTo(hDC, iXIncrement+iLevel, (int)(fHeightRatio * i));
        }

        for(i=64;i<=192;i+=32)
        {
          MoveToEx(hDC, 0, (int)(fHeightRatio * i), NULL);
          LineTo(hDC, iWidth, (int)(fHeightRatio * i));
        }

        SelectObject(hDC,hOldPen);
        DeleteObject(hPen);

        hPen = CreatePen(PS_SOLID, 1, RGB(128, 96, 32));
        hOldPen = (HPEN)SelectObject(hDC,hPen);
        iYIncrement = (int)(fHeightRatio * 192);
        for(i=16;i<=48;i+=16)
        {
          iLevel = (int)(fHeightRatio * i);
          MoveToEx(hDC, 0, iLevel, NULL);
          LineTo(hDC, iWidth, iLevel);
          iLevel += iYIncrement;
          MoveToEx(hDC, 0, iLevel, NULL);
          LineTo(hDC, iWidth, iLevel);
        }
        SelectObject(hDC,hOldPen);
        DeleteObject(hPen);

        sp = (signed short *)g_pScopeBuffer;
        sp += g_iScopeIndex;

        //
        //  Draw old Polyline with background color, to erase it
        //
        hPen = CreatePen(PS_SOLID, 1, RGB(16,96,16));
        hOldPen = (HPEN)SelectObject(hDC,hPen);
        MoveToEx(hDC, 0, (int)((fHeightRatio / 256.0) * *sp), NULL);
        iXIncrement = (AUDIO_BUFFER_SIZE/sizeof(short)/(1000/SCOPEFRAMELENGTH))/iWidth;
        Polyline(hDC, oldScopePoints, iWidth);
        SelectObject(hDC,hOldPen);
        DeleteObject(hPen);

        fHeightRatio /= 256.0;

        //
        //  Draw new Polyline
        //
        hPen = CreatePen(PS_SOLID, 1, RGB(96, 255, 140));
        hOldPen = (HPEN)SelectObject(hDC,hPen);
        MoveToEx(hDC, 0, (int)((fHeightRatio) * *sp), NULL);
        for(i=0;i<iWidth;i++, sp+=iXIncrement )
        {
          newScopePoints[i].x = i;
          newScopePoints[i].y = (int)(((fHeightRatio)) * (*sp + 32768));
        }
        Polyline(hDC, newScopePoints, iWidth);
        memcpy(oldScopePoints,newScopePoints, iWidth * sizeof(POINT));
        SelectObject(hDC,hOldPen);
        DeleteObject(hPen);

        if(fTimerIsPainting)
          g_iScopeIndex += iBufIncrement;
      }
      EndPaint(hwnd, &ps);
      fScopeIsPainting = FALSE;
      break;

    case WM_CREATE:
      pCS = (CREATESTRUCT *)lParam;
      pMDICS = (MDICREATESTRUCT *) pCS->lpCreateParams;
      switch(pMDICS->lParam)
      {
        case MDI_CREATEDATABASEWINDOW:
          g_hDatabaseWindow = hwnd;

          g_hDatabaseListBox = CreateWindow(WC_LISTVIEW, "", WS_CHILD | WS_BORDER |
                               WS_VISIBLE | LVS_REPORT,// | LVS_NOCOLUMNHEADER | LVS_NOSCROLL,
                               0, 0, 0, 0, hwnd, (HMENU)ID_DBLIST, g_hInst,
                               NULL);

          InitializeList(g_hDatabaseListBox);
          break;
        case MDI_CREATESCOPEWINDOW:
          g_hScopeWindow = hwnd;
          break;
        case MDI_CREATEVOLUMEWINDOW:
          g_hVolumeWindow = hwnd;

          g_hMicrophoneVolumeTrackBar = CreateWindow(TRACKBAR_CLASS,"Trackbar",WS_CHILD|
                  WS_BORDER|WS_VISIBLE|WS_TABSTOP|TBS_VERT,
                  0,0,0,0,
                  hwnd,(HMENU)0,g_hInst,NULL);

          g_hOutputVolumeTrackBar = CreateWindow(TRACKBAR_CLASS,"Trackbar",WS_CHILD|
                  WS_BORDER|WS_VISIBLE|WS_TABSTOP|TBS_VERT,
                  0,0,0,0,
                  hwnd,(HMENU)0,g_hInst,NULL);
          
          g_hOutputVolumeLabel = CreateWindowEx(0, "static", "", WS_CHILD | WS_VISIBLE,
                                     0,  0, 0, 0, hwnd, NULL, g_hInst, NULL);
          SendMessage(g_hOutputVolumeLabel, WM_SETFONT, (WPARAM)hLabelFont, MAKELONG(TRUE, 0));
          SetWindowText(g_hOutputVolumeLabel, "Speakers");

          g_hMicrophoneVolumeLabel = CreateWindowEx(0, "static", "", WS_CHILD | WS_VISIBLE,
                                     0,  0, 0, 0, hwnd, NULL, g_hInst, NULL);
          SendMessage(g_hMicrophoneVolumeLabel, WM_SETFONT, (WPARAM)hLabelFont, MAKELONG(TRUE, 0));
          SetWindowText(g_hMicrophoneVolumeLabel, "Microphone");

          g_hAudioDeviceLabel = CreateWindowEx(0, "static", "", WS_CHILD | WS_VISIBLE,
                                     0,  0, 0, 0, hwnd, NULL, g_hInst, NULL);
          SendMessage(g_hAudioDeviceLabel, WM_SETFONT, (WPARAM)hLabelFont, MAKELONG(TRUE, 0));
          SetWindowText(g_hAudioDeviceLabel, "Device");
          
          g_hAudioDeviceComboBox = CreateWindowEx(0, "combobox", NULL,
                           CBS_HASSTRINGS | CBS_DROPDOWNLIST | WS_VSCROLL |
                           WS_TABSTOP | CBS_AUTOHSCROLL |
                           WS_BORDER | WS_CHILD | WS_VISIBLE,
                           115, 195, 100, 100,
                           hwnd, NULL, g_hInst, NULL);

          FillAudioDeviceComboBox();
          GetAudioDeviceID();
/*
          i = SendMessage(g_hAudioDeviceComboBox, CB_FINDSTRING, -1, (DWORD) &g_AudioDeviceIdentifier);
          if(i == CB_ERR)
            i = 0;

          i = SendMessage(g_hAudioDeviceComboBox, CB_SETCURSEL, i, NULL);
          g_iAudioDeviceID = SendMessage(g_hAudioDeviceComboBox, CB_GETITEMDATA, i, NULL);
*/
          break;

        case MDI_CREATELOGWINDOW:
          g_hLogWindow = hwnd;
          g_hLogTextWindow =
          hWndChild = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", "",
                                     WS_CHILD | WS_VISIBLE | WS_VSCROLL |
                                     ES_AUTOVSCROLL | ES_LEFT | ES_MULTILINE |
                                     ES_WANTRETURN | ES_READONLY,
                                     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                                     CW_USEDEFAULT, hwnd, (HMENU)IDC_CHILD_EDIT,
                                     g_hInst, NULL);

          SendMessage(hWndChild, WM_SETFONT, (WPARAM)g_hFont, MAKELONG(TRUE, 0));
          time(&t);
          cp = (unsigned char*)ctime(&t);
          *(cp + strlen((const char*)cp) - 1) = 0;
          WinAFSK_printf(g_hLogTextWindow,"Log window opened on %s\r\n", cp);
          free(cp);
          break;
      }
      if(!hWndChild)
        return 0;
      break;

    case WM_SIZE:
      if(hWndChild = GetDlgItem(hwnd, IDC_CHILD_EDIT))
        MoveWindow(hWndChild, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
      else
        if(hwnd == g_hDatabaseWindow)
        {
          rc.left = 0;
          rc.top = 0;
          rc.right = LOWORD(lParam);
          rc.bottom = HIWORD(lParam);
          MoveWindow(g_hDatabaseListBox, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
        }
        else
          if(hwnd == g_hVolumeWindow)
          {
            fWidth = LOWORD(lParam)/4;
            fHeight = HIWORD(lParam)/3;
            MoveWindow(g_hOutputVolumeTrackBar, (int)fWidth, (int)fHeight*1.5, (int)fWidth/2, (int)fHeight*1.3, TRUE);
            MoveWindow(g_hMicrophoneVolumeTrackBar, (int)fWidth * 2.25, (int)fHeight*1.5, (int)fWidth/2, (int)fHeight*1.3, TRUE);
            MoveWindow(g_hOutputVolumeLabel, (int)fWidth - (44 - fWidth / 2) / 2, (int)fHeight + 8, (int)fWidth*1.5, (int)16, TRUE);
            MoveWindow(g_hMicrophoneVolumeLabel, (int)fWidth * 2.25 - (55 - fWidth / 2) / 2, (int)fHeight + 8, (int)fWidth*1.5, (int)16, TRUE);
            MoveWindow(g_hAudioDeviceComboBox, (int)fWidth/4, (int)24, (int)fWidth*3.5, (int)16, TRUE);
            MoveWindow(g_hAudioDeviceLabel, (int)fWidth/4, (int)6, (int)fWidth*2, (int)16, TRUE);
          }
      break;

    case WM_COMMAND:
      switch(LOWORD(wParam))
      {
        case CM_FILE_SAVE:
          return 0;
    
        case CM_FILE_SAVEAS:
          return 0;

        case CM_EDIT_UNDO:
          SendDlgItemMessage(hwnd, IDC_CHILD_EDIT, EM_UNDO, 0, 0);
          break;

        case CM_EDIT_CUT:
          SendDlgItemMessage(hwnd, IDC_CHILD_EDIT, WM_CUT, 0, 0);
          break;

        case CM_EDIT_COPY:
          SendDlgItemMessage(hwnd, IDC_CHILD_EDIT, WM_COPY, 0, 0);
          break;
  
        case CM_EDIT_PASTE:
          SendDlgItemMessage(hwnd, IDC_CHILD_EDIT, WM_PASTE, 0, 0);
          break;
  
      }
      if(HIWORD(wParam) == CBN_SELCHANGE)
      {
        if(lParam == (long)g_hAudioDeviceComboBox)
        {
          i = SendMessage(g_hAudioDeviceComboBox, CB_GETCURSEL, NULL, NULL);
          //ii = SendMessage(g_hAudioDeviceComboBox, CB_GETLBTEXTLEN, i, NULL);
          //if(ii <= 1024)
          //{
            ii = SendMessage(g_hAudioDeviceComboBox, CB_GETLBTEXT, i, (DWORD) &g_AudioDeviceIdentifier);
            WriteRegistry();
          //}
          SwitchAudioDevice();
        }
      }
      return 0;

    }

  return DefMDIChildProc(hwnd, Message, wParam, lParam);

}

HWND CreateRebar(HWND hWnd)
{
  HWND hRebar;
  REBARINFO rbi;
  
  hRebar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, NULL,
                      WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | RBS_BANDBORDERS |
                      RBS_VARHEIGHT |  WS_CLIPCHILDREN | CCS_NODIVIDER,
                      0, 0, 0, 0, hWnd, (HMENU)ID_REBAR, g_hInst, NULL);

  ZeroMemory(&rbi, sizeof(REBARINFO));
  rbi.cbSize = sizeof(REBARINFO);
  SendMessage(hRebar, RB_SETBARINFO, 0, (LPARAM)&rbi);

  return hRebar;
}

//
//  PrintString(): Appends a string to the contents of the EDIT window.
//
void PrintString(HWND hWnd, char *szText)
{
  int ndx;

  ndx = GetWindowTextLength(hWnd);

  SendMessage (hWnd, EM_SETSEL, (WPARAM)ndx, (LPARAM)ndx);
  SendMessage (hWnd, EM_REPLACESEL, FALSE, (LPARAM) ((LPSTR) szText));
}

// 
//  OpenStream(): Initiates audio input processing. WndProc recieves MM_WIM_*
//                messages whenever a full buffer is ready to be processed.
//                WndProc() calls StreamData() to process the incoming data.
//
BOOL OpenStream(PCMWAVEFORMAT *pwf) 
{ 
  int i;
  short *sp;
  char szBuf[16];

  pInfo = (PLEVELSTREAMDATA) GlobalLock(GlobalAlloc(GHND, sizeof(LEVELSTREAMDATA))); 

  if (pInfo == NULL)  
      return(FALSE); 

  // complete remaining areas of wf 
  pwf->wf.wFormatTag = WAVE_FORMAT_PCM;

  pwf->wf.nBlockAlign = (pwf->wf.nChannels * pwf->wBitsPerSample) / 8 * pwf->wf.nChannels;
  pwf->wf.nAvgBytesPerSec = pwf->wf.nSamplesPerSec * pwf->wf.nBlockAlign; 
  
  pInfo->pwf = pwf; 

  pInfo->buffersize = AUDIO_BUFFER_SIZE; 

  pInfo->hwav = NULL; 


  i = waveInOpen(&pInfo->hwav,g_iAudioDeviceID,(LPWAVEFORMATEX)pwf,
                (DWORD) (UINT) g_hMainWindow, 0, CALLBACK_WINDOW);

  if(i)
  {
//    sprintf(szBuf,"%i",i);
//    MessageBox(NULL,szBuf,"",MB_OK);
    return(FALSE); 
  }

  // set all the wave headers to null (for cleanup if error)
  for (i = 0; i < NUM_AUDIO_BUFFERS; i++)
      pInfo->alpWave[i] = NULL; 

  // alloc, prepare and add all the buffers 
  for (i = 0; i < NUM_AUDIO_BUFFERS; i++)
  {
    pInfo->alpWave[i] = (struct wavehdr_tag *)GlobalLock(GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE, 
                    sizeof(WAVEHDR))); 

    //
    //  An extra space (length = CORRLEN) is required in the buffer for
    //  overlapping data. See the description for the StreamData() function.
    //
    pAudioBuffers[i] = (short *)GlobalLock(GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE,
                       pInfo->buffersize + (CORRLEN * sizeof(short)))); 

    if(pInfo->alpWave[i] == NULL)
      return(FALSE); 

    //
    //  We want the waveIn functions to record starting at position CORRLEN
    //
    sp = pAudioBuffers[i];
    sp += CORRLEN;

    pInfo->alpWave[i]->lpData = (char *) sp;

    pInfo->alpWave[i]->dwBufferLength = pInfo->buffersize; 
    pInfo->alpWave[i]->dwBytesRecorded = 0; 
    pInfo->alpWave[i]->dwUser = 0; 
    pInfo->alpWave[i]->dwFlags = 0; 
    pInfo->alpWave[i]->dwLoops = 0; 

    if(waveInPrepareHeader(pInfo->hwav, pInfo->alpWave[i], sizeof(WAVEHDR)))
      return(FALSE); 

    if(waveInAddBuffer(pInfo->hwav, pInfo->alpWave[i], sizeof(WAVEHDR)))
      return(FALSE); 
  } 

  waveInStart(pInfo->hwav); 
  return(TRUE); 
}
 
// 
//  CloseStream(): Terminates audio input processing.
//
void CloseStream(void) 
{ 
  int i; 

  // stop streaming data 
  waveInStop(pInfo->hwav); 

  // release all buffers 
  waveInReset(pInfo->hwav); 

  // unlock and free buffers 
  for (i = 0; i < NUM_AUDIO_BUFFERS; i++)
    if (pInfo->alpWave[i])
    {
      waveInUnprepareHeader(pInfo->hwav, pInfo->alpWave[i], sizeof(WAVEHDR)); 
      GlobalFree(GlobalHandle(pAudioBuffers[i])); 
      GlobalFree(GlobalHandle(pInfo->alpWave[i])); 
      pInfo->alpWave[i] = NULL; 
    } 

  waveInClose(pInfo->hwav); 
  GlobalFree(GlobalHandle(pInfo)); 
} 

// 
//  StreamData(): Processes the incoming audio buffer. CORRLEN, is equal to
//                18 in the AFSK1200 modulation scheme. The demodulation code
//                requires that the first 18 samples of the buffer are copies
//                of the last 18 samples of the previous buffer. So a
//                temporary static buffer buf[] is used to store the samples.
//
//
BOOL StreamData(HWAVEIN hwav, LPWAVEHDR pHdr) 
{
  static short buf[CORRLEN];
  unsigned short *sp;
  unsigned char *cp;
  time_t t;

  sp = (unsigned short*)pHdr->lpData;
  sp -= CORRLEN;

  if(sp==(unsigned short*)pAudioBuffers[0])
    g_pScopeBuffer = (unsigned short*)pAudioBuffers[1];
  else
    g_pScopeBuffer = (unsigned short*)pAudioBuffers[0];

  g_pScopeBuffer += CORRLEN;

  memcpy(sp, buf, CORRLEN);
  Demodulate((signed short *) sp, pInfo->buffersize / CORRLEN / sizeof(short), CORRLEN);
  memcpy(buf, sp + (pInfo->buffersize/sizeof(short)), CORRLEN);


  //
  // return the buffer back on the waveIn queue
  //
  if ( waveInAddBuffer(pInfo->hwav, pHdr, sizeof(WAVEHDR)) )
    return(FALSE); 

  g_hScopeFrameCount = g_iScopeIndex = 0;
  SetTimer(g_hScopeWindow, IDSCOPETIMER, SCOPEFRAMELENGTH, NULL);

  time( &t );
  cp = (unsigned char*)ctime( &t );
  SendMessage(g_hStatusBar, SB_SETTEXT, 1, (LPARAM)cp);
  free(cp);

  return TRUE;
} 

void OpenLogWindow(void)
{
  MDICREATESTRUCT mcs;
  //
  //  Open the log window
  //
  mcs.szTitle = g_szLogTitle;
  mcs.szClass = g_szLogClass;
  mcs.hOwner  = g_hInst;
  mcs.x = mcs.cx = CW_USEDEFAULT;
  mcs.y = mcs.cy = CW_USEDEFAULT;
  mcs.style = MDIS_ALLCHILDSTYLES;
  mcs.lParam = MDI_CREATELOGWINDOW;
  SendMessage(g_hMDIClient, WM_MDICREATE, 0, (LONG)&mcs);
}

void OpenDatabaseWindow(void)
{
  MDICREATESTRUCT mcs;
  //
  //  Open the scope window
  //
  mcs.szTitle = g_szDatabaseTitle;
  mcs.szClass = g_szDatabaseClass;
  mcs.hOwner  = g_hInst;
  mcs.x = mcs.cx = CW_USEDEFAULT;
  mcs.y = mcs.cy = CW_USEDEFAULT;
  mcs.style = MDIS_ALLCHILDSTYLES;
  mcs.lParam = MDI_CREATEDATABASEWINDOW;
  SendMessage(g_hMDIClient, WM_MDICREATE, 0, (LONG)&mcs);
}

void OpenVolumeWindow(void)
{
  MDICREATESTRUCT mcs;
  //
  //  Open the Volume window
  //
  mcs.szTitle = g_szVolumeTitle;
  mcs.szClass = g_szVolumeClass;
  mcs.hOwner  = g_hInst;
  mcs.x = mcs.cx = CW_USEDEFAULT;
  mcs.y = mcs.cy = CW_USEDEFAULT;
  mcs.style = MDIS_ALLCHILDSTYLES;
  mcs.lParam = MDI_CREATEVOLUMEWINDOW;

  SendMessage(g_hMDIClient, WM_MDICREATE, 0, (LONG)&mcs);

  MixerChange(g_iMicrophoneVolumeMixerControlID);
  MixerChange(g_iOutputVolumeMixerControlID);
}

void OpenScopeWindow(void)
{
  MDICREATESTRUCT mcs;
  //
  //  Open the scope window
  //
  mcs.szTitle = g_szScopeTitle;
  mcs.szClass = g_szScopeClass;
  mcs.hOwner  = g_hInst;
  mcs.x = mcs.cx = CW_USEDEFAULT;
  mcs.y = mcs.cy = CW_USEDEFAULT;
  mcs.style = MDIS_ALLCHILDSTYLES;
  mcs.lParam = MDI_CREATESCOPEWINDOW;
  memset(newScopePoints,0,AUDIO_BUFFER_SIZE);
  memset(oldScopePoints,0,AUDIO_BUFFER_SIZE);
  SendMessage(g_hMDIClient, WM_MDICREATE, 0, (LONG)&mcs);
}

BOOL OpenDatabase(void)
{
  return (g_fpDB = fopen(g_szDatabaseFileName,"a+")) != NULL;
}

void CloseDatabase(void)
{
  fclose(g_fpDB);
}

BOOL GetNextRecord(Record *record)
{
  if(g_fpDB)
  {
    return fread(record,DB_RECORDSIZE,1,g_fpDB) == 1;
  }
  return FALSE;
}

BOOL AddNewRecord(Record *record)
{
  if(g_fpDB)
  {
    fseek(g_fpDB,0,SEEK_END);
    record->Flags = 0;
    return fwrite(record,DB_RECORDSIZE,1,g_fpDB) == DB_RECORDSIZE;
  }
  return NULL;
}

//
//  WinAFSK_printf(): A function with a variable argument list, for printing
//                     to an edit control. Has a fixed maximum string length
//                     defined as EDIT_LENGTH. There is no bounds checking,
//                     so if the string size exceeds that, the program will 
//                     crash.
//
void WinAFSK_printf(HWND hWnd, const char *fmt, ...)
{
  va_list args;

  if(hWnd)
  {
    va_start(args, fmt);
    vsprintf(bufText, fmt, args);
    PrintString(hWnd, bufText);
    va_end(args);
  }
}

HWND CreateToolBar(HWND hWnd)
{
  HWND hTool;
  int wmax, iScopeButton, iLogButton, iDBButton, iVolumeButton, iArrangeButton;
  TBADDBITMAP tbab;
  char szBuf[16];

  wmax = GetSystemMetrics(SM_CXSCREEN);

  hTool = CreateWindowEx(
      0, TOOLBARCLASSNAME, NULL,
      WS_CHILD | CCS_ADJUSTABLE | CCS_NORESIZE | WS_VISIBLE, 
      0, 0, wmax, TOOLBARHEIGHT, hWnd, (HMENU)ID_TOOLBAR, g_hInst, NULL); 

  SendMessage(hTool, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);
  
  tbab.hInst = NULL; 
  tbab.nID = (int)LoadBitmap(g_hInst, "IDB_BMPBUTTONS");
  SendMessage(hTool, TB_ADDBITMAP, (WPARAM)7, (LPARAM)&tbab);

  memset(szBuf,0,sizeof(szBuf));
  strcpy(szBuf,"Scope");
  iScopeButton = SendMessage(hTool, TB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf);

  memset(szBuf,0,sizeof(szBuf));
  strcpy(szBuf,"Log");
  iLogButton = SendMessage(hTool, TB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf);

  memset(szBuf,0,sizeof(szBuf));
  strcpy(szBuf,"Database");
  iDBButton = SendMessage(hTool, TB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf);

  memset(szBuf,0,sizeof(szBuf));
  strcpy(szBuf,"Volume");
  iVolumeButton = SendMessage(hTool, TB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf);

  memset(szBuf,0,sizeof(szBuf));
  strcpy(szBuf,"Arrange");
  iArrangeButton = SendMessage(hTool, TB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf);

  tbb[0].iString = iScopeButton;
  tbb[1].iString = iLogButton;
  tbb[2].iString = iDBButton;
  tbb[3].iString = iVolumeButton;
  // four and five are seperators
  tbb[6].iString = iArrangeButton;
  
  SendMessage(hTool, TB_ADDBUTTONS, (WPARAM)7, (LPARAM)(LPTBBUTTON)&tbb);
  ShowWindow(hTool, SW_SHOW); 
  return hTool;
}

void InitializeList(HWND hList)
{
  LVCOLUMN lc;
  LVITEM li;
  Record record;
  char *cp;

  lc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;//LVCF_WIDTH | LVCF_SUBITEM;
  lc.fmt = LVCFMT_LEFT;

  lc.cx = g_iHeaderWidths[0];
  lc.iSubItem = 0;
  lc.pszText = "Modulation";
  ListView_InsertColumn(hList, 0, &lc);

  lc.cx = g_iHeaderWidths[1];
  lc.iSubItem = 1;
  lc.pszText = "Time";
  ListView_InsertColumn(hList, 1, &lc);

  lc.cx = g_iHeaderWidths[2];
  lc.iSubItem = 2;
  lc.pszText = "From";
  ListView_InsertColumn(hList, 2, &lc);

  lc.cx = g_iHeaderWidths[3];
  lc.iSubItem = 3;
  lc.pszText = "To";
  ListView_InsertColumn(hList, 3, &lc);

  lc.cx = g_iHeaderWidths[4];
  lc.iSubItem = 4;
  lc.pszText = "Message";
  ListView_InsertColumn(hList, 4, &lc);
  
  fseek(g_fpDB,0,SEEK_SET);

  li.mask = LVIF_TEXT;
  while(GetNextRecord(&record))
  {
    li.pszText = record.Modulation;
    li.iItem = 0;
    li.iSubItem = 0;
    ListView_InsertItem(hList, &li);

    cp = ctime(&record.TimeStamp);
    *(cp + strlen(cp) - 1) = 0;
    li.pszText = cp;
    li.iItem = 0;
    li.iSubItem = 1;
    ListView_SetItem(hList, &li);
    free(cp);

    li.pszText = record.From;
    li.iItem = 0;
    li.iSubItem = 2;
    ListView_SetItem(hList, &li);

    li.pszText = record.To;
    li.iItem = 0;
    li.iSubItem = 3;
    ListView_SetItem(hList, &li);

    li.pszText = record.Message;
    li.iItem = 0;
    li.iSubItem = 4;
    ListView_SetItem(hList, &li);

  }
}

void AddDatabaseListItem(Record *record)
{
  char *cp;
  LVITEM li;

  li.mask = LVIF_TEXT;

  li.pszText = record->Modulation;
  li.iItem = 0;
  li.iSubItem = 0;
  ListView_InsertItem(g_hDatabaseListBox, &li);

  cp = ctime(&record->TimeStamp);
  *(cp + strlen(cp) - 1) = 0;
  li.pszText = cp;
  li.iItem = 0;
  li.iSubItem = 1;
  ListView_SetItem(g_hDatabaseListBox, &li);
  free(cp);

  li.pszText = record->From;
  li.iItem = 0;
  li.iSubItem = 2;
  ListView_SetItem(g_hDatabaseListBox, &li);

  li.pszText = record->To;
  li.iItem = 0;
  li.iSubItem = 3;
  ListView_SetItem(g_hDatabaseListBox, &li);

  li.pszText = record->Message;
  li.iItem = 0;
  li.iSubItem = 4;
  ListView_SetItem(g_hDatabaseListBox, &li);
}

void ReadRegistry(DWORD dwMaxLen, unsigned char *pszDirectory)
{
  DWORD dwType;
  unsigned long lRet, lType;
  HKEY hKey;

  pszDirectory[0]=0;

  lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Software\\WinAFSK", 0, KEY_QUERY_VALUE, &hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  lRet = RegQueryValueEx(hKey,"HomeDirectory",0,&lType,pszDirectory,&dwMaxLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  dwType = REG_SZ;
  dwMaxLen = 1023;
  lRet = RegQueryValueEx(hKey,"AudioDevice",0,&lType,(unsigned char*)&g_AudioDeviceIdentifier,&dwMaxLen);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;
  
  RegCloseKey(hKey);

  return;
}

//
//   FUNCTION: CenterWindow(HWND, HWND)
//
//   PURPOSE: Centers one window over another.
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
//       This function will center one window over another ensuring that
//    the placement of the window is within the 'working area', meaning
//    that it is both within the display limits of the screen, and not
//    obscured by the tray or other framing elements of the desktop.
//
BOOL CenterWindow (HWND hwndChild, HWND hwndParent, int addx, int addy)
{
  RECT    rChild, rParent, rWorkArea;
  int     wChild, hChild, wParent, hParent;
  int     xNew, yNew;
  BOOL  bResult;

  // Get the Height and Width of the child window
  GetWindowRect (hwndChild, &rChild);
  wChild = rChild.right - rChild.left + addx;
  hChild = rChild.bottom - rChild.top + addy;

  // Get the Height and Width of the parent window
  //
  // If the parent hWnd is null, center the window on the screen
  //
  if(hwndParent)
  {
    GetWindowRect (hwndParent, &rParent);
    wParent = rParent.right - rParent.left;
    hParent = rParent.bottom - rParent.top;
  }
  else
  {
    rParent.left = rParent.top = 0;
    wParent = rParent.right  = GetSystemMetrics(SM_CXSCREEN);
    hParent = rParent.bottom = GetSystemMetrics(SM_CYSCREEN);
  }

  // Get the limits of the 'workarea'
  bResult = SystemParametersInfo(
     SPI_GETWORKAREA,  // system parameter to query or set
     sizeof(RECT),
     &rWorkArea,
     0);
  if (!bResult)
  {
     rWorkArea.left = rWorkArea.top = 0;
     rWorkArea.right = GetSystemMetrics(SM_CXSCREEN);
     rWorkArea.bottom = GetSystemMetrics(SM_CYSCREEN);
  }

  // Calculate new X position, then adjust for workarea
  xNew = rParent.left + ((wParent - wChild) /2);
  if (xNew < rWorkArea.left)
  {
     xNew = rWorkArea.left;
  } else if ((xNew+wChild) > rWorkArea.right)
  {
     xNew = rWorkArea.right - wChild;
  }

  // Calculate new Y position, then adjust for workarea
  yNew = rParent.top  + ((hParent - hChild) /2);
  if (yNew < rWorkArea.top)
  {
     yNew = rWorkArea.top;
  } else if ((yNew+hChild) > rWorkArea.bottom)
  {
     yNew = rWorkArea.bottom - hChild;
  }

  // Set it, and return
  return SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

void UpdateRebarStatusBox(Record *pr)
{
  char *cp;
  char szBuf[1024];

  cp = ctime(&pr->TimeStamp);

  //
  //  Remove the trailing linefeed that windows ctime() puts on it
  //
  *(cp + strlen(cp) - 1) = 0;

  wsprintf(szBuf,"%s: %s %s to %s\r\n%s", pr->Modulation, cp, pr->From, pr->To, pr->Message);
  SetWindowText(g_hRebarStatusText, (LPSTR) szBuf);
  free(cp);
}

//  SetVolume() Win32 
//
//  SetVolume():  Sets the volume of the audio device. dwValue is a value
//                between 0 and 65535, that indicates the desired volume 
//                level. idComponent is the MIXER_COMPONENTTYPE constant 
//                for the desired component.
//
//                  an example of a few constants;
//                    
//                  MIXERLINE_COMPONENTTYPE_DST_SPEAKERS 
//                  MIXERLINE_COMPONENTTYPE_DST_HEADPHONES
//                  MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE
//                  MIXERLINE_COMPONENTTYPE_SRC_LINE
void SetVolume(DWORD dwValue, int idComponent)
{
  MMRESULT result; 
  HMIXER hMixer;

  MIXERLINE ml = {0}; 
  MIXERLINECONTROLS mlc = {0}; 
  MIXERCONTROL mc = {0}; 
  MIXERCONTROLDETAILS mcd = {0}; 
  MIXERCONTROLDETAILS_UNSIGNED mcdu = {0}; 

  ml.cbStruct = sizeof(MIXERLINE); 
  ml.dwComponentType = idComponent; 
  result = mixerGetLineInfo((HMIXEROBJ) g_hMixer, &ml, MIXER_GETLINEINFOF_COMPONENTTYPE);
  
  mlc.cbStruct = sizeof(MIXERLINECONTROLS); 
  mlc.dwLineID = ml.dwLineID; 
  mlc.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME; 
  mlc.cControls = 1; 
  mlc.pamxctrl = &mc; 
  mlc.cbmxctrl = sizeof(MIXERCONTROL); 
  result = mixerGetLineControls((HMIXEROBJ) g_hMixer, &mlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
  
  mcdu.dwValue = dwValue;

  mcd.cbStruct = sizeof(MIXERCONTROLDETAILS); 
  mcd.hwndOwner = 0; 
  mcd.dwControlID = mc.dwControlID; 
  mcd.paDetails = &mcdu; 
  mcd.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED); 
  mcd.cChannels = 1; 
  result = mixerSetControlDetails((HMIXEROBJ) g_hMixer, &mcd, MIXER_SETCONTROLDETAILSF_VALUE);
}

void OpenMixer(void)
{
  mixerOpen(&g_hMixer, 0, (unsigned long)g_hMainWindow, 0, CALLBACK_WINDOW | MIXER_OBJECTF_MIXER);
  g_iMicrophoneVolumeMixerControlID = GetMixerControlID(MIXERLINE_COMPONENTTYPE_SRC_MICROPHONE);
  g_iOutputVolumeMixerControlID = GetMixerControlID(MIXERLINE_COMPONENTTYPE_DST_SPEAKERS);
}

void MixerChange(unsigned long dwControlID)
{
  unsigned int ui, volume;

  volume = GetVolume(dwControlID);
  ui = volume / 655;
  ui = 100 - ui;

  if(dwControlID == g_iMicrophoneVolumeMixerControlID)
    SendMessage((HWND)g_hMicrophoneVolumeTrackBar, TBM_SETPOS, TRUE, ui);
  else
    if(dwControlID == g_iOutputVolumeMixerControlID)
      SendMessage((HWND)g_hOutputVolumeTrackBar, TBM_SETPOS, TRUE, ui);
}

long GetVolume(unsigned long dwControlID)
{
  MIXERCONTROLDETAILS_UNSIGNED *aDetails;
  MIXERCONTROLDETAILS ControlDetails;
  MMRESULT mmResult;

  aDetails = (MIXERCONTROLDETAILS_UNSIGNED*)malloc(1*sizeof(MIXERCONTROLDETAILS_UNSIGNED));

  memset(&ControlDetails, 0, sizeof(MIXERCONTROLDETAILS));
  ControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
  ControlDetails.dwControlID = dwControlID;
  ControlDetails.cChannels = 1;
  ControlDetails.cMultipleItems = 0;
  ControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
  ControlDetails.paDetails = &aDetails[0];
  mmResult = mixerGetControlDetails( (HMIXEROBJ)g_hMixer, &ControlDetails, MIXER_GETCONTROLDETAILSF_VALUE );
  free( aDetails );
  return aDetails[0].dwValue;
}

int GetMixerControlID(long dwComponentType)
{
  MMRESULT mmResult;
  MIXERLINE MixerLine;
  MIXERCONTROL Control;
  MIXERLINECONTROLS LineControls;

  memset( &MixerLine, 0, sizeof(MIXERLINE) );
  MixerLine.cbStruct = sizeof(MIXERLINE);
  MixerLine.dwComponentType = dwComponentType;
  mmResult = mixerGetLineInfo( (HMIXEROBJ)g_hMixer, &MixerLine, MIXER_GETLINEINFOF_COMPONENTTYPE );

  memset( &Control, 0, sizeof(MIXERCONTROL) );
  Control.cbStruct = sizeof(MIXERCONTROL);

  memset( &LineControls, 0, sizeof(MIXERLINECONTROLS) );
  LineControls.cbStruct = sizeof(MIXERLINECONTROLS);

  LineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
  LineControls.dwLineID = MixerLine.dwLineID;
  LineControls.cControls = 1;
  LineControls.cbmxctrl = sizeof(MIXERCONTROL);
  LineControls.pamxctrl = &Control;
  mmResult = mixerGetLineControls( (HMIXEROBJ)g_hMixer, &LineControls, MIXER_GETLINECONTROLSF_ONEBYTYPE );

  return(Control.dwControlID);
}

void FillAudioDeviceComboBox(void)
{
  int i, iNumDevs;
  WAVEINCAPS w;

  for(iNumDevs = waveInGetNumDevs();iNumDevs > 0;iNumDevs--)
  {
    if(waveInGetDevCaps(iNumDevs - 1, &w, sizeof(w)) == MMSYSERR_NOERROR)
    {
//      if(w.dwFormats & WAVE_FORMAT_2M16)
//      {
        i = SendMessage(g_hAudioDeviceComboBox, CB_ADDSTRING, 0, (DWORD)&w.szPname);
        SendMessage(g_hAudioDeviceComboBox, CB_SETITEMDATA, i, (DWORD)iNumDevs - 1);
//      }
    }
  }
}

void WriteRegistry(void)
{
  HKEY hKey;
  long lRet;

  lRet = RegCreateKey(HKEY_LOCAL_MACHINE, "Software\\WinAFSK",&hKey);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

  lRet = RegSetValueEx(hKey,"AudioDevice",0,REG_SZ,(unsigned char *)g_AudioDeviceIdentifier,1023);
  if((LONG)ERROR_SUCCESS!=lRet)
    return;

//  //debug
//  MessageBox(NULL,g_AudioDeviceIdentifier,"WriteRegistry",MB_OK);

  RegCloseKey(hKey);
}

void SwitchAudioDevice(void)
{

  CloseStream();

  GetAudioDeviceID();

  if(!OpenStream(&waveformat))
    MessageBox(NULL,"Unable to open the selected audio device", g_szAppName,MB_OK);

}

void GetAudioDeviceID()
{
  int i;

  i = SendMessage(g_hAudioDeviceComboBox, CB_FINDSTRING, -1, (DWORD) &g_AudioDeviceIdentifier);
  if(i == CB_ERR)
    i = 0;
  i = SendMessage(g_hAudioDeviceComboBox, CB_SETCURSEL, i, NULL);
  g_iAudioDeviceID = SendMessage(g_hAudioDeviceComboBox, CB_GETITEMDATA, i, NULL);
}
